@@ -341,7 +341,7 @@ Command help:
         print _('Back ends:'), ', '.join(backends)
 
     def do_install(self, tracker_home, args):
-        ""'''Usage: install [template [backend [admin password]]]
+        ""'''Usage: install [template [backend [admin password [key=val[,key=val]]]]]
         Install a new Roundup tracker.
 
         The command will prompt for the tracker home directory
@@ -349,6 +349,13 @@ Command help:
         The template, backend and admin password may be specified
         on the command-line as arguments, in that order.
 
+        The last command line argument allows to pass initial values
+        for config options.  For example, passing
+        "web_http_auth=no,rdbms_user=dinsdale" will override defaults
+        for options http_auth in section [web] and user in section [rdbms].
+        Please be careful to not use spaces in this argument! (Enclose
+        whole argument in quotes if you need spaces in option value).
+
         The initialise command must be called after this command in order
         to initialise the tracker's database. You may edit the tracker's
         initial database contents before running that command by editing
@@ -402,8 +409,18 @@ Erase it? Y/N: """) % locals())
                 backend = 'anydbm'
         # XXX perform a unit test based on the user's selections
 
+        # Process configuration file definitions
+        if len(args) > 3:
+            try:
+                defns = dict([item.split("=") for item in args[3].split(",")])
+            except:
+                print _('Error in configuration settings: "%s"') % args[3]
+                raise
+        else:
+            defns = {}
+
         # install!
-        init.install(tracker_home, templates[template]['path'])
+        init.install(tracker_home, templates[template]['path'], settings=defns)
         init.write_select_db(tracker_home, backend)
 
         print _("""
@@ -1138,16 +1155,19 @@ Erase it? Y/N: """))
             file_props = None
             maxid = 1
             # loop through the file and create a node for each entry
-            for r in reader:
+            for n, r in enumerate(reader):
                 if file_props is None:
                     file_props = r
                     continue
+                sys.stdout.write('Importing %s - %d\r'%(classname, n))
+                sys.stdout.flush()
+
                 # do the import and figure the current highest nodeid
                 nodeid = int(cl.import_list(file_props, r))
                 if hasattr(cl, 'import_files'):
                     cl.import_files(dir, nodeid)
                 maxid = max(maxid, nodeid)
-
+            print
             f.close()
 
             # import the journals
@@ -1221,7 +1241,7 @@ Erase it? Y/N: """))
                     cl = self.get_class(arg)
                     self.db.reindex(arg)
         else:
-            self.db.reindex()
+            self.db.reindex(show_progress=True)
         return 0
 
     def do_security(self, args):
