@@ -41,7 +41,11 @@ from roundup.i18n import _
 
 from blobfiles import FileStorage
 from sessions_dbm import Sessions, OneTimeKeys
-from indexer_dbm import Indexer
+
+try:
+    from indexer_xapian import Indexer
+except ImportError:
+    from indexer_dbm import Indexer
 
 def db_exists(config):
     # check for the user db
@@ -90,7 +94,7 @@ class Database(FileStorage, hyperdb.Database, roundupdb.Database):
         self.newnodes = {}      # keep track of the new nodes by class
         self.destroyednodes = {}# keep track of the destroyed nodes by class
         self.transactions = []
-        self.indexer = Indexer(self.dir)
+        self.indexer = Indexer(self)
         self.security = security.Security(self)
         # ensure files are group readable and writable
         os.umask(0002)
@@ -730,32 +734,6 @@ _marker = []
 class Class(hyperdb.Class):
     '''The handle to a particular class of nodes in a hyperdatabase.'''
 
-    def __init__(self, db, classname, **properties):
-        '''Create a new class with a given name and property specification.
-
-        'classname' must not collide with the name of an existing class,
-        or a ValueError is raised.  The keyword arguments in 'properties'
-        must map names to property objects, or a TypeError is raised.
-        '''
-        for name in 'creation activity creator actor'.split():
-            if properties.has_key(name):
-                raise ValueError, '"creation", "activity", "creator" and '\
-                    '"actor" are reserved'
-
-        self.classname = classname
-        self.properties = properties
-        self.db = weakref.proxy(db)       # use a weak ref to avoid circularity
-        self.key = ''
-
-        # should we journal changes (default yes)
-        self.do_journal = 1
-
-        # do the db-related init stuff
-        db.addclass(self)
-
-        self.auditors = {'create': [], 'set': [], 'retire': [], 'restore': []}
-        self.reactors = {'create': [], 'set': [], 'retire': [], 'restore': []}
-
     def enableJournalling(self):
         '''Turn journalling on for this class
         '''
@@ -1375,31 +1353,6 @@ class Class(hyperdb.Class):
         '''Return the name of the key property for this class or None.'''
         return self.key
 
-    def labelprop(self, default_to_id=0):
-        '''Return the property name for a label for the given node.
-
-        This method attempts to generate a consistent label for the node.
-        It tries the following in order:
-
-        1. key property
-        2. "name" property
-        3. "title" property
-        4. first property from the sorted property name list
-        '''
-        k = self.getkey()
-        if  k:
-            return k
-        props = self.getprops()
-        if props.has_key('name'):
-            return 'name'
-        elif props.has_key('title'):
-            return 'title'
-        if default_to_id:
-            return 'id'
-        props = props.keys()
-        props.sort()
-        return props[0]
-
     # TODO: set up a separate index db file for this? profile?
     def lookup(self, keyvalue):
         '''Locate a particular node by its key property and return its id.
@@ -1428,16 +1381,16 @@ class Class(hyperdb.Class):
 
     # change from spec - allows multiple props to match
     def find(self, **propspec):
-        '''Get the ids of items in this class which link to the given items.
+        '''Get the ids of nodes in this class which link to the given nodes.
 
-        'propspec' consists of keyword args propname=itemid or
-                   propname={<itemid 1>:1, <itemid 2>: 1, ...}
+        'propspec' consists of keyword args propname=nodeid or
+                   propname={nodeid:1, }
         'propname' must be the name of a property in this class, or a
                    KeyError is raised.  That property must be a Link or
                    Multilink property, or a TypeError is raised.
 
-        Any item in this class whose 'propname' property links to any of
-        the itemids will be returned. Examples::
+        Any node in this class whose 'propname' property links to any of
+        the nodeids will be returned. Examples::
 
             db.issue.find(messages='1')
             db.issue.find(messages={'1':1,'3':1}, files={'7':1})
@@ -1535,8 +1488,11 @@ class Class(hyperdb.Class):
         l.sort()
         return l
 
-    def getnodeids(self, db=None):
+    def getnodeids(self, db=None, retired=None):
         ''' Return a list of ALL nodeids
+
+            Set retired=None to get all nodes. Otherwise it'll get all the
+            retired or non-retired nodes, depending on the flag.
         '''
         res = []
 
@@ -1544,16 +1500,31 @@ class Class(hyperdb.Class):
         if self.db.newnodes.has_key(self.classname):
             res += self.db.newnodes[self.classname].keys()
 
+        must_close = False
         if db is None:
             db = self.db.getclassdb(self.classname)
-        res = res + db.keys()
+            must_close = True 
+        try:
+            res = res + db.keys()
 
-        # remove the uncommitted, destroyed nodes
-        if self.db.destroyednodes.has_key(self.classname):
-            for nodeid in self.db.destroyednodes[self.classname].keys():
-                if db.has_key(nodeid):
-                    res.remove(nodeid)
+            # remove the uncommitted, destroyed nodes
+            if self.db.destroyednodes.has_key(self.classname):
+                for nodeid in self.db.destroyednodes[self.classname].keys():
+                    if db.has_key(nodeid):
+                        res.remove(nodeid)
 
+            # check retired flag
+            if retired is False or retired is True:
+                l = []
+                for nodeid in res:
+                    node = self.db.getnode(self.classname, nodeid, db)
+                    is_ret = node.has_key(self.db.RETIRED_FLAG)
+                    if retired == is_ret:
+                        l.append(nodeid)
+                res = l
+        finally:
+            if must_close:
+                db.close()
         return res
 
     def filter(self, search_matches, filterspec, sort=(None,None),
@@ -1620,7 +1591,7 @@ class Class(hyperdb.Class):
                     l.append((STRING, k, re.compile(v, re.I)))
             elif isinstance(propclass, hyperdb.Date):
                 try:
-                    date_rng = propclass.range_from_raw (v, self.db)
+                    date_rng = propclass.range_from_raw(v, self.db)
                     l.append((DATE, k, date_rng))
                 except ValueError:
                     # If range creation fails - ignore that search parameter
@@ -1781,12 +1752,8 @@ class Class(hyperdb.Class):
                         elif isinstance(propclass, hyperdb.Link):
                             lcn = propclass.classname
                             link = self.db.classes[lcn]
-                            key = None
-                            if link.getprops().has_key('order'):
-                                key = 'order'
-                            elif link.getkey():
-                                key = link.getkey()
-                            if key:
+                            key = link.orderprop()
+                            if key!='id':
                                 if not lcache.has_key(v):
                                     # open the link class db if it's not already
                                     if lcldb is None:
@@ -1887,36 +1854,6 @@ class Class(hyperdb.Class):
                     continue
                 self.db.indexer.add_text((self.classname, nodeid, prop), value)
 
-
-    #
-    # Detector interface
-    #
-    def audit(self, event, detector):
-        '''Register a detector
-        '''
-        l = self.auditors[event]
-        if detector not in l:
-            self.auditors[event].append(detector)
-
-    def fireAuditors(self, action, nodeid, newvalues):
-        '''Fire all registered auditors.
-        '''
-        for audit in self.auditors[action]:
-            audit(self.db, self, nodeid, newvalues)
-
-    def react(self, event, detector):
-        '''Register a detector
-        '''
-        l = self.reactors[event]
-        if detector not in l:
-            self.reactors[event].append(detector)
-
-    def fireReactors(self, action, nodeid, oldvalues):
-        '''Fire all registered reactors.
-        '''
-        for react in self.reactors[action]:
-            react(self.db, self, nodeid, oldvalues)
-
     #
     # import / export support
     #
