@@ -46,7 +46,7 @@ import logging
 import metakit
 from sessions_dbm import Sessions, OneTimeKeys
 import re, marshal, os, sys, time, calendar, shutil
-from indexer_dbm import Indexer
+from indexer_common import Indexer
 import locking
 from roundup.date import Range
 from blobfiles import files_in_dir
@@ -82,6 +82,11 @@ def Database(config, journaltag=None):
     return db
 
 class _Database(hyperdb.Database, roundupdb.Database):
+    # Metakit has no concept of an explicit NULL
+    BACKEND_MISSING_STRING = ''
+    BACKEND_MISSING_NUMBER = 0
+    BACKEND_MISSING_BOOLEAN = 0
+
     def __init__(self, config, journaltag=None):
         self.config = config
         self.journaltag = journaltag
@@ -89,7 +94,7 @@ class _Database(hyperdb.Database, roundupdb.Database):
         self.dirty = 0
         self.lockfile = None
         self._db = self.__open()
-        self.indexer = Indexer(self.config.DATABASE, self._db)
+        self.indexer = Indexer(self)
         self.security = security.Security(self)
 
         self.stats = {'cache_hits': 0, 'cache_misses': 0, 'get_items': 0,
@@ -399,42 +404,28 @@ class Class(hyperdb.Class):
 
     privateprops = None
     def __init__(self, db, classname, **properties):
-        if (properties.has_key('creation') or properties.has_key('activity')
-            or properties.has_key('creator') or properties.has_key('actor')):
-            raise ValueError, '"creation", "activity" and "creator" are '\
-                  'reserved'
         if hasattr(db, classname):
             raise ValueError, "Class %s already exists"%classname
 
-        self.db = db
-        self.classname = classname
+        hyperdb.Class.__init__ (self, db, classname, **properties)
+        self.db = db # why isn't this a weakref as for other backends??
         self.key = None
-        self.ruprops = properties
+        self.ruprops = self.properties
         self.privateprops = { 'id' : hyperdb.String(),
                               'activity' : hyperdb.Date(),
                               'actor' : hyperdb.Link('user'),
                               'creation' : hyperdb.Date(),
                               'creator'  : hyperdb.Link('user') }
 
-        # event -> list of callables
-        self.auditors = {'create': [], 'set': [], 'retire': [], 'restore': []}
-        self.reactors = {'create': [], 'set': [], 'retire': [], 'restore': []}
+        self.idcache = {}
+        self.uncommitted = {}
+        self.comactions = []
+        self.rbactions = []
 
         view = self.__getview()
         self.maxid = 1
         if view:
             self.maxid = view[-1].id + 1
-        self.uncommitted = {}
-        self.comactions = []
-        self.rbactions = []
-
-        # people reach inside!!
-        self.properties = self.ruprops
-        self.db.addclass(self)
-        self.idcache = {}
-
-        # default is to journal changes
-        self.do_journal = 1
 
     def setid(self, maxid):
         self.maxid = maxid + 1
@@ -449,35 +440,6 @@ class Class(hyperdb.Class):
         '''
         self.do_journal = 0
 
-    #
-    # Detector/reactor interface
-    #
-    def audit(self, event, detector):
-        '''Register a detector
-        '''
-        l = self.auditors[event]
-        if detector not in l:
-            self.auditors[event].append(detector)
-
-    def fireAuditors(self, action, nodeid, newvalues):
-       '''Fire all registered auditors.
-        '''
-       for audit in self.auditors[action]:
-            audit(self.db, self, nodeid, newvalues)
-
-    def react(self, event, detector):
-       '''Register a reactor
-       '''
-       l = self.reactors[event]
-       if detector not in l:
-           self.reactors[event].append(detector)
-
-    def fireReactors(self, action, nodeid, oldvalues):
-        '''Fire all registered reactors.
-        '''
-        for react in self.reactors[action]:
-            react(self.db, self, nodeid, oldvalues)
-
     # --- the hyperdb.Class methods
     def create(self, **propvalues):
         ''' Create a new node of this class and return its id.
@@ -533,7 +495,7 @@ class Class(hyperdb.Class):
             oldnode = self.uncommitted.get(id, None)
             if oldnode and oldnode.has_key(propname):
                 raw = oldnode[propname]
-                converter = _converters.get(rutyp.__class__, None)
+                converter = _converters.get(raw.__class__, None)
                 if converter:
                     return converter(raw)
                 return raw
@@ -1082,16 +1044,16 @@ class Class(hyperdb.Class):
             self.db.dirty = 1
 
     def find(self, **propspec):
-        '''Get the ids of items in this class which link to the given items.
+        '''Get the ids of nodes in this class which link to the given nodes.
 
-        'propspec' consists of keyword args propname=itemid or
-                   propname={<itemid 1>:1, <itemid 2>: 1, ...}
+        'propspec' consists of keyword args propname=nodeid or
+                   propname={nodeid:1, }
         'propname' must be the name of a property in this class, or a
                    KeyError is raised.  That property must be a Link or
                    Multilink property, or a TypeError is raised.
 
-        Any item in this class whose 'propname' property links to any of
-        the itemids will be returned. Examples::
+        Any node in this class whose 'propname' property links to any of
+        the nodeids will be returned. Examples::
 
             db.issue.find(messages='1')
             db.issue.find(messages={'1':1,'3':1}, files={'7':1})
@@ -1156,14 +1118,18 @@ class Class(hyperdb.Class):
             l.append(str(row.id))
         return l
 
-    def getnodeids(self):
+    def getnodeids(self, retired=None):
         ''' Retrieve all the ids of the nodes for a particular Class.
 
             Set retired=None to get all nodes. Otherwise it'll get all the
             retired or non-retired nodes, depending on the flag.
         '''
         l = []
-        for row in self.getview():
+        if retired is False or retired is True:
+            result = self.getview().select(_isdel=retired)
+        else:
+            result = self.getview()
+        for row in result:
             l.append(str(row.id))
         return l
 
@@ -1386,11 +1352,7 @@ class Class(hyperdb.Class):
                     lv = linkclass.getview()
                     lv = lv.rename('id', propname)
                     v = v.join(lv, prop, 1)
-                    if linkclass.getprops().has_key('order'):
-                        propname = 'order'
-                    else:
-                        propname = linkclass.labelprop()
-                    prop = getattr(v, propname)
+                    prop = getattr(v, linkclass.orderprop())
                 if isreversed:
                     rev.append(prop)
                 sortspec.append(prop)
@@ -1415,31 +1377,6 @@ class Class(hyperdb.Class):
         '''
         return int(nodeid) < self.maxid
 
-    def labelprop(self, default_to_id=0):
-        '''Return the property name for a label for the given node.
-
-        This method attempts to generate a consistent label for the node.
-        It tries the following in order:
-
-        1. key property
-        2. "name" property
-        3. "title" property
-        4. first property from the sorted property name list
-        '''
-        k = self.getkey()
-        if  k:
-            return k
-        props = self.getprops()
-        if props.has_key('name'):
-            return 'name'
-        elif props.has_key('title'):
-            return 'title'
-        if default_to_id:
-            return 'id'
-        props = props.keys()
-        props.sort()
-        return props[0]
-
     def stringFind(self, **requirements):
         '''Locate a particular node by matching a set of its String
         properties in a caseless search.
@@ -1696,6 +1633,7 @@ class Class(hyperdb.Class):
 
         No heading here - the columns are fixed.
         '''
+        from roundup.hyperdb import Interval, Date, Password
         properties = self.getprops()
         r = []
         for nodeid in self.getnodeids():
@@ -1878,6 +1816,14 @@ class FileClass(hyperdb.FileClass, Class):
             os.makedirs(os.path.dirname(dest))
         shutil.copyfile(source, dest)
 
+        mime_type = None
+        if self.getprops().has_key('type'):
+            mime_type = self.get(nodeid, 'type')
+        if not mime_type:
+            mime_type = self.default_mime_type
+        self.db.indexer.add_text((self.classname, nodeid, 'content'),
+            self.get(nodeid, 'content'), mime_type)
+
     def get(self, nodeid, propname, default=_marker, cache=1):
         if propname == 'content':
             poss_msg = 'Possibly an access right configuration problem.'
@@ -2004,12 +1950,11 @@ class IssueClass(Class, roundupdb.IssueClass):
 
 CURVERSION = 2
 
-class Indexer(Indexer):
-    disallows = {'THE':1, 'THIS':1, 'ZZZ':1, 'THAT':1, 'WITH':1}
-    def __init__(self, path, datadb):
-        self.path = os.path.join(path, 'index.mk4')
+class MetakitIndexer(Indexer):
+    def __init__(self, db):
+        self.path = os.path.join(db.config.DATABASE, 'index.mk4')
         self.db = metakit.storage(self.path, 1)
-        self.datadb = datadb
+        self.datadb = db._db
         self.reindex = 0
         v = self.db.view('version')
         if not v.structure():
@@ -2082,7 +2027,7 @@ class Indexer(Indexer):
         wordlist = re.findall(r'\b\w{2,25}\b', text.upper())
         words = {}
         for word in wordlist:
-            if not self.disallows.has_key(word):
+            if not self.is_stopword(word):
                 words[word] = 1
         words = words.keys()
 
@@ -2116,8 +2061,8 @@ class Indexer(Indexer):
             if len(hits) == 0:
                 return {}
         if hits is None:
-            return {}
-        rslt = {}
+            return []
+        rslt = []
         ids = self.db.view('ids').remapwith(hits)
         tbls = self.datadb.view('tables')
         for i in range(len(ids)):
@@ -2126,7 +2071,7 @@ class Indexer(Indexer):
                 classname = tbls[hit.tblid].name
                 nodeid = str(hit.nodeid)
                 property = self._getpropname(classname, hit.propid)
-                rslt[i] = (classname, nodeid, property)
+                rslt.append((classname, nodeid, property))
         return rslt
 
     def save_index(self):
@@ -2140,4 +2085,9 @@ class Indexer(Indexer):
             self.db = metakit.storage(self.path, 1)
         self.changed = 0
 
+try:
+    from indexer_xapian import Indexer
+except ImportError:
+    Indexer = MetakitIndexer
+
 # vim: set et sts=4 sw=4 :
