@@ -1,3 +1,4 @@
+#$Id$
 #
 # Copyright (c) 2003 Martynas Sklyzmantas, Andrey Lebedev <andrey@micro.lt>
 #
@@ -116,7 +117,7 @@ class Database(Database):
     #mysql_backend = 'BDB'
 
     hyperdb_to_sql_datatypes = {
-        hyperdb.String : 'VARCHAR(255)',
+        hyperdb.String : 'TEXT',
         hyperdb.Date   : 'DATETIME',
         hyperdb.Link   : 'INTEGER',
         hyperdb.Interval  : 'VARCHAR(255)',
@@ -165,7 +166,7 @@ class Database(Database):
             if message[0] != ER.NO_SUCH_TABLE:
                 raise DatabaseError, message
             self.init_dbschema()
-            self.sql("CREATE TABLE schema (schema TEXT) TYPE=%s"%
+            self.sql("CREATE TABLE `schema` (`schema` TEXT) TYPE=%s"%
                 self.mysql_backend)
             self.sql('''CREATE TABLE ids (name VARCHAR(255),
                 num INTEGER) TYPE=%s'''%self.mysql_backend)
@@ -388,6 +389,38 @@ class Database(Database):
         self.create_class_table_indexes(spec)
         return cols, mls
 
+    def create_class_table_indexes(self, spec):
+        ''' create the class table for the given spec
+        '''
+        # create __retired__ index
+        index_sql2 = 'create index _%s_retired_idx on _%s(__retired__)'%(
+                        spec.classname, spec.classname)
+        self.sql(index_sql2)
+
+        # create index for key property
+        if spec.key:
+            if isinstance(spec.properties[spec.key], String):
+                idx = spec.key + '(255)'
+            else:
+                idx = spec.key
+            index_sql3 = 'create index _%s_%s_idx on _%s(_%s)'%(
+                        spec.classname, spec.key,
+                        spec.classname, idx)
+            self.sql(index_sql3)
+
+        # TODO: create indexes on (selected?) Link property columns, as
+        # they're more likely to be used for lookup
+
+    def create_class_table_key_index(self, cn, key):
+        ''' create the class table for the given spec
+        '''
+        prop = self.classes[cn].getprops()[key]
+        if isinstance(prop, String):
+            sql = 'create index _%s_%s_idx on _%s(_%s(255))'%(cn, key, cn, key)
+        else:
+            sql = 'create index _%s_%s_idx on _%s(_%s)'%(cn, key, cn, key)
+        self.sql(sql)
+
     def drop_class_table_indexes(self, cn, key):
         # drop the old table indexes first
         l = ['_%s_id_idx'%cn, '_%s_retired_idx'%cn]
@@ -690,6 +723,7 @@ class MysqlClass:
         orderby = []
         ordercols = []
         mlsort = []
+        rhsnum = 0
         for sortby in group, sort:
             sdir, prop = sortby
             if sdir and prop:
@@ -705,14 +739,15 @@ class MysqlClass:
                     lcn = props[prop].classname
                     link = self.db.classes[lcn]
                     o = '_%s._%s'%(cn, prop)
-                    if link.getprops().has_key('order'):
+                    op = link.orderprop ()
+                    if op != 'id':
                         tn = '_' + lcn
-                        loj.append('LEFT OUTER JOIN %s on %s=%s.id'%(tn,
-                            o, tn))
-                        ordercols.append(tn + '._order')
-                        o = tn + '._order'
-                    else:
-                        ordercols.append(o)
+                        rhs = 'rhs%s_'%rhsnum
+                        rhsnum += 1
+                        loj.append('LEFT OUTER JOIN %s as %s on %s=%s.id'%(
+                            tn, rhs, o, rhs))
+                        o = '%s._%s'%(rhs, op)
+                    ordercols.append(o)
                 elif prop == 'id':
                     o = '_%s.id'%cn
                 else:
