@@ -1,4 +1,21 @@
-# $Id$
+#
+# Copyright (c) 2001 Bizar Software Pty Ltd (http://www.bizarsoftware.com.au/)
+# This module is free software, and you may redistribute it and/or modify
+# under the same terms as Python, so long as this copyright message and
+# disclaimer are retained in their original form.
+#
+# IN NO EVENT SHALL BIZAR SOFTWARE PTY LTD BE LIABLE TO ANY PARTY FOR
+# DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
+# OUT OF THE USE OF THIS CODE, EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE
+# POSSIBILITY OF SUCH DAMAGE.
+#
+# BIZAR SOFTWARE PTY LTD SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
+# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+# FOR A PARTICULAR PURPOSE.  THE CODE PROVIDED HEREUNDER IS ON AN "AS IS"
+# BASIS, AND THERE IS NO OBLIGATION WHATSOEVER TO PROVIDE MAINTENANCE,
+# SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
+#
+#$Id$
 ''' Relational database (SQL) backend common code.
 
 Basics:
@@ -32,14 +49,17 @@ __docformat__ = 'restructuredtext'
 import sys, os, time, re, errno, weakref, copy, logging
 
 # roundup modules
-from roundup import hyperdb, date, password, roundupdb, security
+from roundup import hyperdb, date, password, roundupdb, security, support
 from roundup.hyperdb import String, Password, Date, Interval, Link, \
     Multilink, DatabaseError, Boolean, Number, Node
 from roundup.backends import locking
 
 # support
 from blobfiles import FileStorage
-from indexer_rdbms import Indexer
+try:
+    from indexer_xapian import Indexer
+except ImportError:
+    from indexer_rdbms import Indexer
 from sessions_rdbms import Sessions, OneTimeKeys
 from roundup.date import Range
 
@@ -178,9 +198,9 @@ class Database(FileStorage, hyperdb.Database, roundupdb.Database):
             We should now confirm that the schema defined by our "classes"
             attribute actually matches the schema in the database.
         '''
-        save = self.upgrade_db()
+        save = 0
 
-        # now detect changes in the schema
+        # handle changes in the schema
         tables = self.database_schema['tables']
         for classname, spec in self.classes.items():
             if tables.has_key(classname):
@@ -199,6 +219,10 @@ class Database(FileStorage, hyperdb.Database, roundupdb.Database):
                 del tables[classname]
                 save = 1
 
+        # now upgrade the database for column type changes, new internal
+        # tables, etc.
+        save = save | self.upgrade_db()
+
         # update the database version of the schema
         if save:
             self.save_dbschema()
@@ -313,14 +337,19 @@ class Database(FileStorage, hyperdb.Database, roundupdb.Database):
         self.post_init()
 
 
-    def reindex(self, classname=None):
+    def reindex(self, classname=None, show_progress=False):
         if classname:
             classes = [self.getclass(classname)]
         else:
             classes = self.classes.values()
         for klass in classes:
-            for nodeid in klass.list():
-                klass.index(nodeid)
+            if show_progress:
+                for nodeid in support.Progress('Reindex %s'%klass.classname,
+                        klass.list()):
+                    klass.index(nodeid)
+            else:
+                for nodeid in klass.list():
+                    klass.index(nodeid)
         self.indexer.save_index()
 
     hyperdb_to_sql_datatypes = {
@@ -389,7 +418,10 @@ class Database(FileStorage, hyperdb.Database, roundupdb.Database):
         # detect key prop change for potential index change
         keyprop_changes = {}
         if new_spec[0] != old_spec[0]:
-            keyprop_changes = {'remove': old_spec[0], 'add': new_spec[0]}
+            if old_spec[0]:
+                keyprop_changes['remove'] = old_spec[0]
+            if new_spec[0]:
+                keyprop_changes['add'] = new_spec[0]
 
         # detect multilinks that have been removed, and drop their table
         old_has = {}
@@ -455,14 +487,21 @@ class Database(FileStorage, hyperdb.Database, roundupdb.Database):
 
         return 1
 
-    def create_class_table(self, spec):
-        '''Create the class table for the given Class "spec". Creates the
-        indexes too.'''
+    def determine_all_columns(self, spec):
+        """Figure out the columns from the spec and also add internal columns
+
+        """
         cols, mls = self.determine_columns(spec.properties.items())
 
         # add on our special columns
         cols.append(('id', 'INTEGER PRIMARY KEY'))
         cols.append(('__retired__', 'INTEGER DEFAULT 0'))
+        return cols, mls
+
+    def create_class_table(self, spec):
+        '''Create the class table for the given Class "spec". Creates the
+        indexes too.'''
+        cols, mls = self.determine_all_columns(spec)
 
         # create the base table
         scols = ','.join(['%s %s'%x for x in cols])
@@ -1170,7 +1209,7 @@ class Database(FileStorage, hyperdb.Database, roundupdb.Database):
         for method, args in self.transactions:
             method(*args)
 
-        # save the indexer state
+        # save the indexer
         self.indexer.save_index()
 
         # clear out the transactions
@@ -1219,32 +1258,6 @@ class Class(hyperdb.Class):
         concrete backend Class.
     '''
 
-    def __init__(self, db, classname, **properties):
-        '''Create a new class with a given name and property specification.
-
-        'classname' must not collide with the name of an existing class,
-        or a ValueError is raised.  The keyword arguments in 'properties'
-        must map names to property objects, or a TypeError is raised.
-        '''
-        for name in 'creation activity creator actor'.split():
-            if properties.has_key(name):
-                raise ValueError, '"creation", "activity", "creator" and '\
-                    '"actor" are reserved'
-
-        self.classname = classname
-        self.properties = properties
-        self.db = weakref.proxy(db)       # use a weak ref to avoid circularity
-        self.key = ''
-
-        # should we journal changes (default yes)
-        self.do_journal = 1
-
-        # do the db-related init stuff
-        db.addclass(self)
-
-        self.auditors = {'create': [], 'set': [], 'retire': [], 'restore': []}
-        self.reactors = {'create': [], 'set': [], 'retire': [], 'restore': []}
-
     def schema(self):
         ''' A dumpable version of the schema that we can store in the
             database
@@ -1837,31 +1850,6 @@ class Class(hyperdb.Class):
         '''Return the name of the key property for this class or None.'''
         return self.key
 
-    def labelprop(self, default_to_id=0):
-        '''Return the property name for a label for the given node.
-
-        This method attempts to generate a consistent label for the node.
-        It tries the following in order:
-
-        1. key property
-        2. "name" property
-        3. "title" property
-        4. first property from the sorted property name list
-        '''
-        k = self.getkey()
-        if  k:
-            return k
-        props = self.getprops()
-        if props.has_key('name'):
-            return 'name'
-        elif props.has_key('title'):
-            return 'title'
-        if default_to_id:
-            return 'id'
-        props = props.keys()
-        props.sort()
-        return props[0]
-
     def lookup(self, keyvalue):
         '''Locate a particular node by its key property and return its id.
 
@@ -1890,16 +1878,16 @@ class Class(hyperdb.Class):
         return str(row[0])
 
     def find(self, **propspec):
-        '''Get the ids of items in this class which link to the given items.
+        '''Get the ids of nodes in this class which link to the given nodes.
 
-        'propspec' consists of keyword args propname=itemid or
-                   propname={<itemid 1>:1, <itemid 2>: 1, ...}
+        'propspec' consists of keyword args propname=nodeid or
+                   propname={nodeid:1, }
         'propname' must be the name of a property in this class, or a
                    KeyError is raised.  That property must be a Link or
                    Multilink property, or a TypeError is raised.
 
-        Any item in this class whose 'propname' property links to any of
-        the itemids will be returned. Examples::
+        Any node in this class whose 'propname' property links to any of
+        the nodeids will be returned. Examples::
 
             db.issue.find(messages='1')
             db.issue.find(messages={'1':1,'3':1}, files={'7':1})
@@ -2142,7 +2130,7 @@ class Class(hyperdb.Class):
                 else:
                     try:
                         # Try to filter on range of dates
-                        date_rng = propclass.range_from_raw (v, self.db)
+                        date_rng = propclass.range_from_raw(v, self.db)
                         if date_rng.from_value:
                             where.append('_%s._%s >= %s'%(cn, k, a))
                             args.append(dc(date_rng.from_value))
@@ -2198,6 +2186,7 @@ class Class(hyperdb.Class):
         orderby = []
         ordercols = []
         mlsort = []
+        rhsnum = 0
         for sortby in group, sort:
             sdir, prop = sortby
             if sdir and prop:
@@ -2213,14 +2202,15 @@ class Class(hyperdb.Class):
                     lcn = props[prop].classname
                     link = self.db.classes[lcn]
                     o = '_%s._%s'%(cn, prop)
-                    if link.getprops().has_key('order'):
+                    op = link.orderprop()
+                    if op != 'id':
                         tn = '_' + lcn
-                        loj.append('LEFT OUTER JOIN %s on %s=%s.id'%(tn,
-                            o, tn))
-                        ordercols.append(tn + '._order')
-                        o = tn + '._order'
-                    else:
-                        ordercols.append(o)
+                        rhs = 'rhs%s_'%rhsnum
+                        rhsnum += 1
+                        loj.append('LEFT OUTER JOIN %s as %s on %s=%s.id'%(
+                            tn, rhs, o, rhs))
+                        o = '%s._%s'%(rhs, op)
+                    ordercols.append(o)
                 elif prop == 'id':
                     o = '_%s.id'%cn
                 else:
@@ -2334,36 +2324,6 @@ class Class(hyperdb.Class):
                 self.db.indexer.add_text((self.classname, nodeid, prop),
                     str(self.get(nodeid, prop)))
 
-
-    #
-    # Detector interface
-    #
-    def audit(self, event, detector):
-        '''Register a detector
-        '''
-        l = self.auditors[event]
-        if detector not in l:
-            self.auditors[event].append(detector)
-
-    def fireAuditors(self, action, nodeid, newvalues):
-        '''Fire all registered auditors.
-        '''
-        for audit in self.auditors[action]:
-            audit(self.db, self, nodeid, newvalues)
-
-    def react(self, event, detector):
-        '''Register a detector
-        '''
-        l = self.reactors[event]
-        if detector not in l:
-            self.reactors[event].append(detector)
-
-    def fireReactors(self, action, nodeid, oldvalues):
-        '''Fire all registered reactors.
-        '''
-        for react in self.reactors[action]:
-            react(self.db, self, nodeid, oldvalues)
-
     #
     # import / export support
     #
@@ -2439,15 +2399,17 @@ class Class(hyperdb.Class):
                 pwd = password.Password()
                 pwd.unpack(value)
                 value = pwd
-            d[propname] = value
-            if isinstance(prop, String):
-                if type(value) != type('') and type(value) != type(u''):
+            elif isinstance(prop, String):
+                if isinstance(value, unicode):
+                    value = value.encode('utf8')
+                if not isinstance(value, str):
                     raise TypeError, \
                         'new property "%(propname)s" not a string: %(value)r' \
                         % locals()
                 if prop.indexme:
                     self.db.indexer.add_text((self.classname, newid, propname),
                         value)
+            d[propname] = value
 
         # get a new id if necessary
         if newid is None:
@@ -2482,7 +2444,12 @@ class Class(hyperdb.Class):
             for nodeid, date, user, action, params in self.history(nodeid):
                 date = date.get_tuple()
                 if action == 'set':
+                    export_data = {}
                     for propname, value in params.items():
+                        if not properties.has_key(propname):
+                            # property no longer in the schema
+                            continue
+
                         prop = properties[propname]
                         # make sure the params are eval()'able
                         if value is None:
@@ -2493,7 +2460,8 @@ class Class(hyperdb.Class):
                             value = value.get_tuple()
                         elif isinstance(prop, Password):
                             value = str(value)
-                        params[propname] = value
+                        export_data[propname] = value
+                    params = export_data
                 l = [nodeid, date, user, action, params]
                 r.append(map(repr, l))
         return r
