@@ -1,6 +1,6 @@
 #$Id$
 
-import re, cgi, StringIO, urllib, Cookie, time, random, csv
+import re, cgi, StringIO, urllib, Cookie, time, random, csv, codecs
 
 from roundup import hyperdb, token, date, password
 from roundup.i18n import _
@@ -148,6 +148,12 @@ class SearchAction(Action):
         """
         self.fakeFilterVars()
         queryname = self.getQueryName()
+    
+        # editing existing query name?
+        old_queryname = ''
+        for key in ('@old-queryname', ':old-queryname'):
+            if self.form.has_key(key):
+                old_queryname = self.form[key].value.strip()
 
         # handle saving the query params
         if queryname:
@@ -162,7 +168,7 @@ class SearchAction(Action):
             if key:
                 # edit the old way, only one query per name
                 try:
-                    qid = self.db.query.lookup(queryname)
+                    qid = self.db.query.lookup(old_queryname)
                     if not self.hasPermission('Edit', 'query', itemid=qid):
                         raise exceptions.Unauthorised, self._(
                             "You do not have permission to edit queries")
@@ -178,23 +184,24 @@ class SearchAction(Action):
                 # edit the new way, query name not a key any more
                 # see if we match an existing private query
                 uid = self.db.getuid()
-                qids = self.db.query.filter(None, {'name': queryname,
+                qids = self.db.query.filter(None, {'name': old_queryname,
                         'private_for': uid})
                 if not qids:
                     # ok, so there's not a private query for the current user
-                    # - see if there's a public one created by them
-                    qids = self.db.query.filter(None, {'name': queryname,
-                        'private_for': -1, 'creator': uid})
+                    # - see if there's one created by them
+                    qids = self.db.query.filter(None, {'name': old_queryname,
+                        'creator': uid})
 
-                if qids:
+                if qids and old_queryname:
                     # edit query - make sure we get an exact match on the name
                     for qid in qids:
-                        if queryname != self.db.query.get(qid, 'name'):
+                        if old_queryname != self.db.query.get(qid, 'name'):
                             continue
                         if not self.hasPermission('Edit', 'query', itemid=qid):
                             raise exceptions.Unauthorised, self._(
                             "You do not have permission to edit queries")
-                        self.db.query.set(qid, klass=self.classname, url=url)
+                        self.db.query.set(qid, klass=self.classname,
+                            url=url, name=queryname)
                 else:
                     # create a query
                     if not self.hasPermission('Create', 'query'):
@@ -239,10 +246,9 @@ class SearchAction(Action):
 
             self.form.value.append(cgi.MiniFieldStorage('@filter', key))
 
-    FV_QUERYNAME = re.compile(r'[@:]queryname')
     def getQueryName(self):
-        for key in self.form.keys():
-            if self.FV_QUERYNAME.match(key):
+        for key in ('@queryname', ':queryname'):
+            if self.form.has_key(key):
                 return self.form[key].value.strip()
         return ''
 
@@ -729,7 +735,7 @@ class RegoCommon(Action):
                 user=user, last_use=time.time())
         else:
             # new session cookie
-            self.client.set_cookie(user)
+            self.client.set_cookie(user, expire=None)
 
         # nice message
         message = self._('You are now registered, welcome!')
@@ -822,9 +828,10 @@ class RegisterAction(RegoCommon, EditCommon):
         # send the email
         tracker_name = self.db.config.TRACKER_NAME
         tracker_email = self.db.config.TRACKER_EMAIL
-        subject = 'Complete your registration to %s -- key %s'%(tracker_name,
+        if self.db.config['EMAIL_REGISTRATION_CONFIRMATION']:
+            subject = 'Complete your registration to %s -- key %s'%(tracker_name,
                                                                   otk)
-        body = """To complete your registration of the user "%(name)s" with
+            body = """To complete your registration of the user "%(name)s" with
 %(tracker)s, please do one of the following:
 
 - send a reply to %(tracker_email)s and maintain the subject line as is (the
@@ -836,8 +843,17 @@ reply's additional "Re:" is ok),
 
 """ % {'name': user_props['username'], 'tracker': tracker_name,
         'url': self.base, 'otk': otk, 'tracker_email': tracker_email}
+        else:
+            subject = 'Complete your registration to %s'%(tracker_name)
+            body = """To complete your registration of the user "%(name)s" with
+%(tracker)s, please visit the following URL:
+
+%(url)s?@action=confrego&otk=%(otk)s
+
+""" % {'name': user_props['username'], 'tracker': tracker_name,
+        'url': self.base, 'otk': otk}
         if not self.client.standard_message([user_props['address']], subject,
-                body, (tracker_name, tracker_email)):
+                body):
             return
 
         # commit changes to the database
@@ -897,7 +913,14 @@ class LoginAction(Action):
         self.client.opendb(self.client.user)
 
         # set the session cookie
-        self.client.set_cookie(self.client.user)
+        if self.form.has_key('remember'):
+            self.client.set_cookie(self.client.user, expire=86400*365)
+        else:
+            self.client.set_cookie(self.client.user, expire=None)
+
+        # If we came from someplace, go back there
+        if self.form.has_key('__came_from'):
+            raise exceptions.Redirect, self.form['__came_from'].value
 
     def verifyLogin(self, username, password):
         # make sure the user exists
@@ -947,7 +970,7 @@ class ExportCSVAction(Action):
             matches = None
 
         h = self.client.additional_headers
-        h['Content-Type'] = 'text/csv'
+        h['Content-Type'] = 'text/csv; charset=%s' % self.client.charset
         # some browsers will honor the filename here...
         h['Content-Disposition'] = 'inline; filename=query.csv'
 
@@ -957,7 +980,12 @@ class ExportCSVAction(Action):
             # all done, return a dummy string
             return 'dummy'
 
-        writer = csv.writer(self.client.request.wfile)
+        wfile = self.client.request.wfile
+        if self.client.charset != self.client.STORAGE_CHARSET:
+            wfile = codecs.EncodedFile(wfile,
+                self.client.STORAGE_CHARSET, self.client.charset, 'replace')
+
+        writer = csv.writer(wfile)
         writer.writerow(columns)
 
         # and search
