@@ -15,6 +15,7 @@ from roundup.exceptions import *
 from roundup.cgi.exceptions import *
 from roundup.cgi.form_parser import FormParser
 from roundup.mailer import Mailer, MessageSendError
+from roundup.cgi import accept_language
 
 def initialiseSecurity(security):
     '''Create some Permissions and Roles on the security object
@@ -27,6 +28,7 @@ def initialiseSecurity(security):
     security.addPermissionToRole('Admin', p)
 
     # doing Role stuff through the web - make sure Admin can
+    # TODO: deprecate this and use a property-based control
     p = security.addPermission(name="Web Roles",
         description="User may manipulate user Roles through the web")
     security.addPermissionToRole('Admin', p)
@@ -44,6 +46,12 @@ def clean_message_callback(match, ok={'a':1,'i':1,'b':1,'br':1}):
         return match.group(1)
     return '&lt;%s&gt;'%match.group(2)
 
+error_message = ""'''<html><head><title>An error has occurred</title></head>
+<body><h1>An error has occurred</h1>
+<p>A problem was encountered processing your request.
+The tracker maintainers have been notified of the problem.</p>
+</body></html>'''
+
 class Client:
     '''Instantiate to handle one CGI request.
 
@@ -195,13 +203,15 @@ class Client:
 
         The most common requests are handled like so:
 
-        1. figure out who we are, defaulting to the "anonymous" user
+        1. look for charset and language preferences, set up user locale
+           see determine_charset, determine_language
+        2. figure out who we are, defaulting to the "anonymous" user
            see determine_user
-        2. figure out what the request is for - the context
+        3. figure out what the request is for - the context
            see determine_context
-        3. handle any requested action (item edit, search, ...)
+        4. handle any requested action (item edit, search, ...)
            see handle_action
-        4. render a template, resulting in HTML output
+        5. render a template, resulting in HTML output
 
         In some situations, exceptions occur:
 
@@ -224,6 +234,7 @@ class Client:
         self.error_message = []
         try:
             self.determine_charset()
+            self.determine_language()
 
             # make sure we're identified (even anonymously)
             self.determine_user()
@@ -297,8 +308,11 @@ class Client:
             self.error_message.append(self._('Form Error: ') + str(e))
             self.write_html(self.renderContext())
         except:
-            # everything else
-            self.write_html(cgitb.html(i18n=self.translator))
+            if self.instance.config.WEB_DEBUG:
+                self.write_html(cgitb.html(i18n=self.translator))
+            else:
+                self.mailer.exception_message()
+                return self.write_html(self._(error_message))
 
     def clean_sessions(self):
         """Age sessions, remove when they haven't been used for a week.
@@ -330,7 +344,6 @@ class Client:
         If the charset is found, and differs from the storage charset,
         recode all form fields of type 'text/plain'
         """
-
         # look for client charset
         charset_parameter = 0
         if self.form.has_key('@charset'):
@@ -385,25 +398,46 @@ class Client:
                     value = re_charref.sub(_decode_charref, value)
                     field.value = encoder(value)[0]
 
+    def determine_language(self):
+        """Determine the language"""
+        # look for language parameter
+        # then for language cookie
+        # last for the Accept-Language header
+        if self.form.has_key("@language"):
+            language = self.form["@language"].value
+            if language.lower() == "none":
+                language = ""
+            self.add_cookie("roundup_language", language)
+        elif self.cookie.has_key("roundup_language"):
+            language = self.cookie["roundup_language"].value
+        elif self.instance.config["WEB_USE_BROWSER_LANGUAGE"]:
+            hal = self.env.get('HTTP_ACCEPT_LANGUAGE')
+            language = accept_language.parse(hal)
+        else:
+            language = ""
+
+        self.language = language
+        if language:
+            self.setTranslator(TranslationService.get_translation(
+                    language,
+                    tracker_home=self.instance.config["TRACKER_HOME"]))
+
     def determine_user(self):
-        ''' Determine who the user is
-        '''
-        # determine the uid to use
+        """Determine who the user is"""
         self.opendb('admin')
 
         # make sure we have the session Class
         self.clean_sessions()
         sessions = self.db.getSessionManager()
 
-        # first up, try the REMOTE_USER var (from HTTP Basic Auth handled
-        # by a front-end HTTP server)
-        use_http_auth = self.instance.config['WEB_HTTP_AUTH'] == 'yes'
-        user = 'anonymous'
-        if use_http_auth:
+        user = None
+        # first up, try http authorization if enabled
+        if self.instance.config['WEB_HTTP_AUTH']:
             if self.env.has_key('REMOTE_USER'):
+                # we have external auth (e.g. by Apache)
                 user = self.env['REMOTE_USER']
-            # try handling Basic Auth ourselves
             elif self.env.get('HTTP_AUTHORIZATION', ''):
+                # try handling Basic Auth ourselves
                 auth = self.env['HTTP_AUTHORIZATION']
                 scheme, challenge = auth.split(' ', 1)
                 if scheme.lower() == 'basic':
@@ -423,13 +457,11 @@ class Client:
 
                     user = username
 
-        # look up the user session cookie (may override the HTTP Basic Auth)
-        cookie = self.cookie
-        if (cookie.has_key(self.cookie_name) and
-                cookie[self.cookie_name].value != 'deleted'):
-
+        # if user was not set by http authorization, try session cookie
+        if (not user) and self.cookie.has_key(self.cookie_name) \
+        and (self.cookie[self.cookie_name].value != 'deleted'):
             # get the session key from the cookie
-            self.session = cookie[self.cookie_name].value
+            self.session = self.cookie[self.cookie_name].value
             # get the user from the session
             try:
                 # update the lifetime datestamp
@@ -439,8 +471,13 @@ class Client:
                 # not valid, ignore id
                 pass
 
-        # sanity check on the user still being valid, getting the userid
-        # at the same time
+        # if no user name set by http authorization or session cookie
+        # the user is anonymous
+        if not user:
+            user = 'anonymous'
+
+        # sanity check on the user still being valid,
+        # getting the userid at the same time
         try:
             self.userid = self.db.user.lookup(user)
         except (KeyError, TypeError):
@@ -459,13 +496,13 @@ class Client:
         self.opendb(self.user)
 
     def opendb(self, username):
-        ''' Open the database and set the current user.
+        """Open the database and set the current user.
 
         Opens a database once. On subsequent calls only the user is set on
         the database object the instance.optimize is set. If we are in
         "Development Mode" (cf. roundup_server) then the database is always
         re-opened.
-        '''
+        """
         # don't do anything if the db is open and the user has not changed
         if hasattr(self, 'db') and self.db.isCurrentUser(username):
             return
@@ -826,9 +863,10 @@ class Client:
         for entry in headers.items():
             self.request.send_header(*entry)
         for ((path, name), (value, expire)) in self.add_cookies.items():
-            self.request.send_header('Set-Cookie',
-                "%s=%s; expires=%s; Path=%s;"
-                % (name, value, Cookie._getdate(expire), path))
+            cookie = "%s=%s; Path=%s;"%(name, value, path)
+            if expire is not None:
+                cookie += " expires=%s;"%Cookie._getdate(expire)
+            self.request.send_header('Set-Cookie', cookie)
         self.request.end_headers()
         self.headers_done = 1
         if self.debug:
@@ -847,6 +885,7 @@ class Client:
                 If value is empty (meaning "delete cookie"),
                 expiration time is forced in the past
                 and this argument is ignored.
+                If None, the cookie will expire at end-of-session.
                 If omitted, the cookie will be kept for a year.
             path:
                 cookie path (optional)
@@ -858,7 +897,7 @@ class Client:
             expire = -1
         self.add_cookies[(path, name)] = (value, expire)
 
-    def set_cookie(self, user):
+    def set_cookie(self, user, expire=None):
         """Set up a session cookie for the user.
 
         Also store away the user's login info against the session.
@@ -885,7 +924,7 @@ class Client:
         self.db.commit()
 
         # add session cookie
-        self.add_cookie(self.cookie_name, self.session)
+        self.add_cookie(self.cookie_name, self.session, expire=expire)
 
     def make_user_anonymous(self):
         ''' Make us anonymous
