@@ -20,6 +20,7 @@ __docformat__ = 'restructuredtext'
 from __future__ import nested_scopes
 
 import sys, cgi, urllib, os, re, os.path, time, errno, mimetypes, csv
+import calendar
 
 from roundup import hyperdb, date, support
 from roundup import i18n
@@ -34,9 +35,12 @@ try:
 except ImportError:
     import StringIO
 try:
-    import StructuredText
+    from StructuredText.StructuredText import HTML as StructuredText
 except ImportError:
-    StructuredText = None
+    try: # older version
+        import StructuredText
+    except ImportError:
+        StructuredText = None
 
 # bring in the templating support
 from roundup.cgi.PageTemplates import PageTemplate, GlobalTranslationService
@@ -364,15 +368,13 @@ def lookupIds(db, prop, ids, fail_ok=0, num_re=re.compile('^-?\d+$')):
     cl = db.getclass(prop.classname)
     l = []
     for entry in ids:
-        if num_re.match(entry):
-            l.append(entry)
-        else:
-            try:
-                l.append(cl.lookup(entry))
-            except (TypeError, KeyError):
-                if fail_ok:
-                    # pass through the bad value
-                    l.append(entry)
+        try:
+            l.append(cl.lookup(entry))
+        except (TypeError, KeyError):
+            # if fail_ok, ignore lookup error
+            # otherwise entry must be existing object id rather than key value
+            if fail_ok or num_re.match(entry):
+                l.append(entry)
     return l
 
 def lookupKeys(linkcl, key, ids, num_re=re.compile('^-?\d+$')):
@@ -624,7 +626,7 @@ class HTMLClass(HTMLInputMixin, HTMLPermissions):
 
     def classhelp(self, properties=None, label=''"(list)", width='500',
             height='400', property='', form='itemSynopsis',
-            pagesize=50, sort=None, filter=None):
+            pagesize=50, inputtype="checkbox", sort=None, filter=None):
         '''Pop up a javascript window with class help
 
         This generates a link to a popup window which displays the
@@ -648,6 +650,9 @@ class HTMLClass(HTMLInputMixin, HTMLPermissions):
         If the "property" arg is given, it's passed through to the
         javascript help_window function.
 
+        You can use inputtype="radio" to display a radio box instead
+        of the default checkbox (useful for entering Link-properties)
+
         If the "form" arg is given, it's passed through to the
         javascript help_window function. - it's the name of the form
         the "property" belongs to.
@@ -667,6 +672,8 @@ class HTMLClass(HTMLInputMixin, HTMLPermissions):
             property = '&amp;property=%s'%property
         if form:
             form = '&amp;form=%s'%form
+        if inputtype:
+            type= '&amp;type=%s'%inputtype
         if filter:
             filterprops = filter.split(';')
             filtervalues = []
@@ -679,8 +686,9 @@ class HTMLClass(HTMLInputMixin, HTMLPermissions):
         else:
            filter = ''
         help_url = "%s?@startwith=0&amp;@template=help&amp;"\
-                   "properties=%s%s%s%s&amp;@pagesize=%s%s" % \
-                   (self.classname, properties, property, form, sort, pagesize, filter)
+                   "properties=%s%s%s%s%s&amp;@pagesize=%s%s" % \
+                   (self.classname, properties, property, form, type,
+                   sort, pagesize, filter)
         onclick = "javascript:help_window('%s', '%s', '%s');return false;" % \
                   (help_url, width, height)
         return '<a class="classhelp" href="%s" onclick="%s">%s</a>' % \
@@ -1074,6 +1082,29 @@ class _HTMLItem(HTMLInputMixin, HTMLPermissions):
         url = '%s%s/%s'%(self._classname, self._nodeid, name)
         return urllib.quote(url)
 
+    def copy_url(self, exclude=("messages", "files")):
+        """Construct a URL for creating a copy of this item
+
+        "exclude" is an optional list of properties that should
+        not be copied to the new object.  By default, this list
+        includes "messages" and "files" properties.  Note that
+        "id" property cannot be copied.
+
+        """
+        exclude = ("id", "activity", "actor", "creation", "creator") \
+            + tuple(exclude)
+        query = {
+            "@template": "item",
+            "@note": self._("Copy of %(class)s %(id)s") % {
+                "class": self._(self._classname), "id": self._nodeid},
+        }
+        for name in self._props.keys():
+            if name not in exclude:
+                query[name] = self[name].plain()
+        return self._classname + "?" + "&".join(
+            ["%s=%s" % (key, urllib.quote(value))
+                for key, value in query.items()])
+
 class _HTMLUser(_HTMLItem):
     '''Add ability to check for permissions on users.
     '''
@@ -1087,9 +1118,16 @@ class _HTMLUser(_HTMLItem):
         '''
         if classname is self._marker:
             classname = self._client.classname
-        return self._client.db.security.hasPermission(permission,
+        return self._db.security.hasPermission(permission,
             self._nodeid, classname, property, itemid)
 
+    def hasRole(self, rolename):
+        '''Determine whether the user has the Role.'''
+        roles = self._db.user.get(self._nodeid, 'roles').split(',')
+        for role in roles:
+            if role.strip() == rolename: return True
+        return False
+
 def HTMLItem(client, classname, nodeid, anonymous=0):
     if classname == 'user':
         return _HTMLUser(client, classname, nodeid, anonymous)
@@ -1211,7 +1249,7 @@ class StringHTMLProperty(HTMLProperty):
             s = self.hyper_re.sub(self._hyper_repl, s)
         return s
 
-    def stext(self, escape=0):
+    def stext(self, escape=0, hyperlink=1):
         ''' Render the value of the property as StructuredText.
 
             This requires the StructureText module to be installed separately.
@@ -1219,12 +1257,12 @@ class StringHTMLProperty(HTMLProperty):
         if not self.is_view_ok():
             return self._('[hidden]')
 
-        s = self.plain(escape=escape)
+        s = self.plain(escape=escape, hyperlink=hyperlink)
         if not StructuredText:
             return s
         return StructuredText(s,level=1,header=0)
 
-    def field(self, size = 30):
+    def field(self, **kwargs):
         ''' Render the property as a field in HTML.
 
             If not editable, just display the value via plain().
@@ -1238,7 +1276,10 @@ class StringHTMLProperty(HTMLProperty):
             value = cgi.escape(str(self._value))
 
         value = '&quot;'.join(value.split('"'))
-        return self.input(name=self._formname,value=value,size=size)
+
+        kwargs.setdefault("size", 30)
+        kwargs.update({"name": self._formname, "value": value})
+        return self.input(**kwargs)
 
     def multiline(self, escape=0, rows=5, cols=40):
         ''' Render a multiline form edit field for the property.
@@ -1267,8 +1308,9 @@ class StringHTMLProperty(HTMLProperty):
             value = ''
         else:
             value = str(self._value)
-        if value.find('@') != -1:
-            name, domain = value.split('@')
+        split = value.split('@')
+        if len(split) == 2:
+            name, domain = split
             domain = ' '.join(domain.split('.')[:-1])
             name = name.replace('.', ' ')
             value = '%s at %s ...'%(name, domain)
@@ -1526,12 +1568,12 @@ class DateHTMLProperty(HTMLProperty):
         '''
         if not self.is_view_ok():
             return self._('[hidden]')
-        
+
         if self._offset is None:
             offset = self._db.getUserTimezone()
         else:
             offset = self._offset
-        
+
         if not self._value:
             return ''
         elif format is not self._marker:
@@ -1548,6 +1590,21 @@ class DateHTMLProperty(HTMLProperty):
         return DateHTMLProperty(self._client, self._classname, self._nodeid,
             self._prop, self._formname, self._value, offset=offset)
 
+    def popcal(self, width=300, height=200, label= "(cal)",
+            form="itemSynopsis"):
+        """Generate a link to a calendar pop-up window.
+
+        item: HTMLProperty e.g.: context.deadline
+        """
+        if self.isset():
+            date = "&date=%s"%self._value
+        else :
+            date = ""
+        return ('<a class="classhelp" href="javascript:help_window('
+            "'%s?@template=calendar&property=%s&form=%s%s', %d, %d)"
+            '">%s</a>'%(self._classname, self._name, form, date, width,
+            height, label))
+
 class IntervalHTMLProperty(HTMLProperty):
     def __init__(self, client, classname, nodeid, prop, name, value,
             anonymous=0):
@@ -1658,13 +1715,14 @@ class LinkHTMLProperty(HTMLProperty):
         return '<input name="%s" value="%s" size="%s">'%(self._formname,
             value, size)
 
-    def menu(self, size=None, height=None, showid=0, additional=[],
+    def menu(self, size=None, height=None, showid=0, additional=[], value=None,
             sort_on=None, **conditions):
         ''' Render a form select list for this property
 
             "size" is used to limit the length of the list labels
             "height" is used to set the <select> tag's "size" attribute
             "showid" includes the item ids in the list labels
+            "value" specifies which item is pre-selected
             "additional" lists properties which should be included in the
                 label
             "sort_on" indicates the property to sort the list on as
@@ -1681,7 +1739,8 @@ class LinkHTMLProperty(HTMLProperty):
         if not self.is_edit_ok():
             return self.plain()
 
-        value = self._value
+        if value is None:
+            value = self._value
 
         linkcl = self._db.getclass(self._prop.classname)
         l = ['<select name="%s">'%self._formname]
@@ -1700,7 +1759,10 @@ class LinkHTMLProperty(HTMLProperty):
         else:
             sort_on = ('+', find_sort_key(linkcl))
 
-        options = linkcl.filter(None, conditions, sort_on, (None, None))
+        options = [opt
+            for opt in linkcl.filter(None, conditions, sort_on, (None, None))
+            if self._db.security.hasPermission("View", self._client.userid,
+                linkcl.classname, itemid=id)]
 
         # make sure we list the current value if it's retired
         if value and value not in options:
@@ -1846,7 +1908,7 @@ class MultilinkHTMLProperty(HTMLProperty):
         return self.input(name=self._formname,size=size,value=value)
 
     def menu(self, size=None, height=None, showid=0, additional=[],
-            sort_on=None, **conditions):
+             value=None, sort_on=None, **conditions):
         ''' Render a form <select> list for this property.
 
             "size" is used to limit the length of the list labels
@@ -1854,6 +1916,7 @@ class MultilinkHTMLProperty(HTMLProperty):
             "showid" includes the item ids in the list labels
             "additional" lists properties which should be included in the
                 label
+            "value" specifies which item is pre-selected
             "sort_on" indicates the property to sort the list on as
                 (direction, property) where direction is '+' or '-'. A
                 single string with the direction prepended may be used.
@@ -1868,7 +1931,8 @@ class MultilinkHTMLProperty(HTMLProperty):
         if not self.is_edit_ok():
             return self.plain()
 
-        value = self._value
+        if value is None:
+            value = self._value
 
         linkcl = self._db.getclass(self._prop.classname)
 
@@ -1881,7 +1945,10 @@ class MultilinkHTMLProperty(HTMLProperty):
         else:
             sort_on = ('+', find_sort_key(linkcl))
 
-        options = linkcl.filter(None, conditions, sort_on)
+        options = [opt
+            for opt in linkcl.filter(None, conditions, sort_on)
+            if self._db.security.hasPermission("View", self._client.userid,
+                linkcl.classname, itemid=id)]
         height = height or min(len(options), 7)
         l = ['<select multiple name="%s" size="%s">'%(self._formname, height)]
         k = linkcl.labelprop(1)
@@ -1982,6 +2049,9 @@ class HTMLRequest(HTMLInputMixin):
     - "filterspec" values to filter the index on
     - "search_text" text to perform a full-text search on for an index
     '''
+    def __repr__(self):
+        return '<HTMLRequest %r>'%self.form
+
     def __init__(self, client):
         # _client is needed by HTMLInputMixin
         self._client = self.client = client
@@ -1994,6 +2064,7 @@ class HTMLRequest(HTMLInputMixin):
 
         # store the current class name and action
         self.classname = client.classname
+        self.nodeid = client.nodeid
         self.template = client.template
 
         # the special char to use for special vars
@@ -2003,6 +2074,17 @@ class HTMLRequest(HTMLInputMixin):
 
         self._post_init()
 
+    def current_url(self):
+        url = self.base
+        if self.classname:
+            url += self.classname
+            if self.nodeid:
+                url += self.nodeid
+        args = {}
+        if self.template:
+            args['@template'] = self.template
+        return self.indexargs_url(url, args)
+
     def _post_init(self):
         ''' Set attributes based on self.form
         '''
@@ -2091,6 +2173,11 @@ class HTMLRequest(HTMLInputMixin):
                 self.special_char = name[0]
                 self.startwith = int(self.form[name].value)
 
+        # dispname
+        self.dispname = None
+        if self.form.has_key('@dispname'):
+            self.dispname = self.form['@dispname'].value
+
     def updateFromURL(self, url):
         ''' Parse the URL for query args, and update my attributes using the
             values.
@@ -2401,6 +2488,86 @@ class TemplatingUtils:
             raise AttributeError, name
         return self.client.instance.templating_utils[name]
 
+    def html_calendar(self, request):
+        """Generate a HTML calendar.
+
+        `request`  the roundup.request object
+                   - @template : name of the template
+                   - form      : name of the form to store back the date
+                   - property  : name of the property of the form to store
+                                 back the date
+                   - date      : current date
+                   - display   : when browsing, specifies year and month
+
+        html will simply be a table.
+        """
+        date_str  = request.form.getfirst("date", ".")
+        display   = request.form.getfirst("display", date_str)
+        template  = request.form.getfirst("@template", "calendar")
+        form      = request.form.getfirst("form")
+        property  = request.form.getfirst("property")
+        curr_date = date.Date(date_str) # to highlight
+        display   = date.Date(display)  # to show
+        day       = display.day
+
+        # for navigation
+        date_prev_month = display + date.Interval("-1m")
+        date_next_month = display + date.Interval("+1m")
+        date_prev_year  = display + date.Interval("-1y")
+        date_next_year  = display + date.Interval("+1y")
+
+        res = []
+
+        base_link = "%s?@template=%s&property=%s&form=%s&date=%s" % \
+                    (request.classname, template, property, form, curr_date)
+
+        # navigation
+        # month
+        res.append('<table class="calendar"><tr><td>')
+        res.append(' <table width="100%" class="calendar_nav"><tr>')
+        link = "&display=%s"%date_prev_month
+        res.append('  <td><a href="%s&display=%s">&lt;</a></td>'%(base_link,
+            date_prev_month))
+        res.append('  <td>%s</td>'%calendar.month_name[display.month])
+        res.append('  <td><a href="%s&display=%s">&gt;</a></td>'%(base_link,
+            date_next_month))
+        # spacer
+        res.append('  <td width="100%"></td>')
+        # year
+        res.append('  <td><a href="%s&display=%s">&lt;</a></td>'%(base_link,
+            date_prev_year))
+        res.append('  <td>%s</td>'%display.year)
+        res.append('  <td><a href="%s&display=%s">&gt;</a></td>'%(base_link,
+            date_next_year))
+        res.append(' </tr></table>')
+        res.append(' </td></tr>')
+
+        # the calendar
+        res.append(' <tr><td><table class="calendar_display">')
+        res.append('  <tr class="weekdays">')
+        for day in calendar.weekheader(3).split():
+            res.append('   <td>%s</td>'%day)
+        res.append('  </tr>')
+        for week in calendar.monthcalendar(display.year, display.month):
+            res.append('  <tr>')
+            for day in week:
+                link = "javascript:form[field].value = '%d-%02d-%02d'; " \
+                      "window.close ();"%(display.year, display.month, day)
+                if (day == curr_date.day and display.month == curr_date.month
+                        and display.year == curr_date.year):
+                    # highlight
+                    style = "today"
+                else :
+                    style = ""
+                if day:
+                    res.append('   <td class="%s"><a href="%s">%s</a></td>'%(
+                        style, link, day))
+                else :
+                    res.append('   <td></td>')
+            res.append('  </tr>')
+        res.append('</table></td></tr></table>')
+        return "\n".join(res)
+
 class MissingValue:
     def __init__(self, description, **kwargs):
         self.__description = description
