@@ -268,6 +268,18 @@ class BooleanOption(Option):
             _val = value and 1 or 0
         return _val
 
+class WordListOption(Option):
+
+    """List of strings"""
+
+    class_description = "Allowed values: comma-separated list of words"
+
+    def _value2str(self, value):
+        return ','.join(value)
+
+    def str2value(self, value):
+        return value.split(',')
+
 class RunDetectorOption(Option):
 
     """When a detector is run: always, never or for new items only"""
@@ -448,6 +460,12 @@ SETTINGS = (
         (BooleanOption, "instant_registration", "no",
             "Register new users instantly, or require confirmation via\n"
             "email?"),
+        (BooleanOption, "email_registration_confirmation", "yes",
+            "Offer registration confirmation by email or only through the web?"),
+        (WordListOption, "indexer_stopwords", "",
+            "Additional stop-words for the full-text indexer specific to\n"
+            "your tracker. See the indexer source for the default list of\n"
+            "stop-words (eg. A,AND,ARE,AS,AT,BE,BUT,BY, ...)"),
     )),
     ("tracker", (
         (Option, "name", "Roundup issue tracker",
@@ -468,12 +486,22 @@ SETTINGS = (
             "or LANG, in that order of preference."),
     )),
     ("web", (
-        (Option, 'http_auth', "yes",
+        (BooleanOption, 'http_auth', "yes",
             "Whether to use HTTP Basic Authentication, if present.\n"
             "Roundup will use either the REMOTE_USER or HTTP_AUTHORIZATION\n"
             "variables supplied by your web server (in that order).\n"
             "Set this option to 'no' if you do not wish to use HTTP Basic\n"
             "Authentication in your web interface."),
+        (BooleanOption, 'use_browser_language', "yes",
+            "Whether to use HTTP Accept-Language, if present.\n"
+            "Browsers send a language-region preference list.\n"
+            "It's usually set in the client's browser or in their\n"
+            "Operating System.\n"
+            "Set this option to 'no' if you want to ignore it."),
+        (BooleanOption, "debug", "no",
+            "Setting this option makes Roundup display error tracebacks\n"
+            "in the user's browser rather than emailing them to the\n"
+            "tracker admin."),
     )),
     ("rdbms", (
         (Option, 'name', 'roundup',
@@ -569,6 +597,33 @@ SETTINGS = (
             "Default locale name for the tracker mail gateway.\n"
             "If this option is not set, mail gateway will use\n"
             "the language of the tracker instance."),
+        (Option, "subject_prefix_parsing", "strict",
+            "Controls the parsing of the [prefix] on subject\n"
+            "lines in incoming emails. \"strict\" will return an\n"
+            "error to the sender if the [prefix] is not recognised.\n"
+            "\"loose\" will attempt to parse the [prefix] but just\n"
+            "pass it through as part of the issue title if not\n"
+            "recognised. \"none\" will always pass any [prefix]\n"
+            "through as part of the issue title."),
+        (Option, "subject_suffix_parsing", "strict",
+            "Controls the parsing of the [suffix] on subject\n"
+            "lines in incoming emails. \"strict\" will return an\n"
+            "error to the sender if the [suffix] is not recognised.\n"
+            "\"loose\" will attempt to parse the [suffix] but just\n"
+            "pass it through as part of the issue title if not\n"
+            "recognised. \"none\" will always pass any [suffix]\n"
+            "through as part of the issue title."),
+        (Option, "subject_suffix_delimiters", "[]",
+            "Defines the brackets used for delimiting the commands\n"
+            "suffix in a subject line."),
+        (Option, "subject_content_match", "always",
+            "Controls matching of the incoming email subject line\n"
+            "against issue titles in the case where there is no\n"
+            "designator [prefix]. \"never\" turns off matching.\n"
+            "\"creation + interval\" or \"activity + interval\"\n"
+            "will match an issue for the interval after the issue's\n"
+            "creation or last activity. The interval is a standard\n"
+            "Roundup interval."),
     ), "Roundup Mail Gateway options"),
     ("nosy", (
         (RunDetectorOption, "messages_to_author", "no",
@@ -592,6 +647,11 @@ SETTINGS = (
             "If 'yes', then the recipients will be added on followups too.\n"
             "If 'no', they're never added to the nosy.\n",
             ["ADD_RECIPIENTS_TO_NOSY"]),
+        (Option, "email_sending", "single",
+            "Controls the email sending from the nosy reactor. If\n"
+            "\"multiple\" then a separate email is sent to each\n"
+            "recipient. If \"single\" then a single email is sent with\n"
+            "each recipient as a CC address."),
     ), "Nosy messages sending"),
 )
 
@@ -624,7 +684,7 @@ class Config:
     # actual name of the config file.  set on load.
     filepath = os.path.join(HOME, INI_FILE)
 
-    def __init__(self, config_path=None, layout=None):
+    def __init__(self, config_path=None, layout=None, settings={}):
         """Initialize confing instance
 
         Parameters:
@@ -636,6 +696,9 @@ class Config:
             layout:
                 optional configuration layout, a sequence of
                 section definitions suitable for .add_section()
+            settings:
+                optional setting overrides (dictionary).
+                The overrides are applied after loading config file.
 
         """
         # initialize option containers:
@@ -649,6 +712,8 @@ class Config:
                 self.add_section(*section)
         if config_path is not None:
             self.load(config_path)
+        for (name, value) in settings.items():
+            self[name.upper()] = value
 
     def add_section(self, section, options, description=None):
         """Define new config section
@@ -1041,8 +1106,8 @@ class CoreConfig(Config):
     ext = None
     detectors = None
 
-    def __init__(self, home_dir=None):
-        Config.__init__(self, home_dir, SETTINGS)
+    def __init__(self, home_dir=None, settings={}):
+        Config.__init__(self, home_dir, layout=SETTINGS, settings=settings)
         # load the config if home_dir given
         if home_dir is None:
             self.init_logging()
