@@ -22,7 +22,6 @@
 __docformat__ = 'restructuredtext'
 
 import time, re, calendar
-from types import *
 import i18n
 
 try:
@@ -47,7 +46,7 @@ date_re = re.compile(r'''^
     |(?P<a>\d\d?)[/-](?P<b>\d\d?))?              # or mm-dd
     (?P<n>\.)?                                   # .
     (((?P<H>\d?\d):(?P<M>\d\d))?(:(?P<S>\d\d(\.\d+)?))?)?  # hh:mm:ss
-    (?P<o>[\d\smyd\-+]+)?                        # offset
+    (?P<o>[\d\smywd\-+]+)?                       # offset
 $''', re.VERBOSE)
 serialised_date_re = re.compile(r'''
     (\d{4})(\d\d)(\d\d)(\d\d)(\d\d)(\d\d?(\.\d+)?)
@@ -489,9 +488,9 @@ class Interval:
     ):
         """Construct an interval given a specification."""
         self.setTranslator(translator)
-        if type(spec) in (IntType, FloatType, LongType):
+        if isinstance(spec, (int, float, long)):
             self.from_seconds(spec)
-        elif type(spec) in (StringType, UnicodeType):
+        elif isinstance(spec, basestring):
             self.set(spec, allowdate=allowdate, add_granularity=add_granularity)
         elif isinstance(spec, Interval):
             (self.sign, self.year, self.month, self.day, self.hour,
