@@ -22,10 +22,11 @@
 __docformat__ = 'restructuredtext'
 
 # standard python modules
-import sys, os, time, re, shutil
+import sys, os, time, re, shutil, weakref
 
 # roundup modules
 import date, password
+from support import ensureParentsExist, PrioList
 
 #
 # Types
@@ -82,9 +83,9 @@ class Date:
             raise HyperdbValueError, 'property %s: %r is an invalid '\
                 'date (%s)'%(kw['propname'], value, message)
         return value
-    def range_from_raw (self, value, db):
+    def range_from_raw(self, value, db):
         """return Range value from given raw value with offset correction"""
-        return date.Range(value, date.Date, offset=self.offset (db))
+        return date.Range(value, date.Date, offset=self.offset(db))
 
 class Interval:
     """An object designating an Interval property."""
@@ -270,6 +271,10 @@ All methods except __repr__ must be implemented by a concrete backend Database.
     # flag to set on retired entries
     RETIRED_FLAG = '__hyperdb_retired'
 
+    BACKEND_MISSING_STRING = None
+    BACKEND_MISSING_NUMBER = None
+    BACKEND_MISSING_BOOLEAN = None
+
     def __init__(self, config, journaltag=None):
         """Open a hyperdatabase given a specifier to some storage.
 
@@ -440,7 +445,25 @@ class Class:
         or a ValueError is raised.  The keyword arguments in 'properties'
         must map names to property objects, or a TypeError is raised.
         """
-        raise NotImplementedError
+        for name in 'creation activity creator actor'.split():
+            if properties.has_key(name):
+                raise ValueError, '"creation", "activity", "creator" and '\
+                    '"actor" are reserved'
+
+        self.classname = classname
+        self.properties = properties
+        self.db = weakref.proxy(db)       # use a weak ref to avoid circularity
+        self.key = ''
+
+        # should we journal changes (default yes)
+        self.do_journal = 1
+
+        # do the db-related init stuff
+        db.addclass(self)
+
+        actions = "create set retire restore".split ()
+        self.auditors = dict ([(a, PrioList ()) for a in actions])
+        self.reactors = dict ([(a, PrioList ()) for a in actions])
 
     def __repr__(self):
         '''Slightly more useful representation
@@ -590,6 +613,22 @@ class Class:
         """
         raise NotImplementedError
 
+    def setlabelprop (self, labelprop):
+        """Set the label property. Used for override of labelprop
+           resolution order.
+        """
+        if labelprop not in self.getprops () :
+            raise ValueError, "Not a property name: %s" % labelprop
+        self._labelprop = labelprop
+
+    def setorderprop (self, orderprop):
+        """Set the order property. Used for override of orderprop
+           resolution order
+        """
+        if orderprop not in self.getprops () :
+            raise ValueError, "Not a property name: %s" % orderprop
+        self._orderprop = orderprop
+
     def getkey(self):
         """Return the name of the key property for this class or None."""
         raise NotImplementedError
@@ -600,12 +639,45 @@ class Class:
         This method attempts to generate a consistent label for the node.
         It tries the following in order:
 
+        0. self._labelprop if set
         1. key property
         2. "name" property
         3. "title" property
         4. first property from the sorted property name list
         """
-        raise NotImplementedError
+        if hasattr (self, '_labelprop') :
+            return self._labelprop
+        k = self.getkey()
+        if  k:
+            return k
+        props = self.getprops()
+        if props.has_key('name'):
+            return 'name'
+        elif props.has_key('title'):
+            return 'title'
+        if default_to_id:
+            return 'id'
+        props = props.keys()
+        props.sort()
+        return props[0]
+
+    def orderprop (self):
+        """Return the property name to use for sorting for the given node.
+
+        This method computes the property for sorting.
+        It tries the following in order:
+
+        0. self._orderprop if set
+        1. "order" property
+        2. self.labelprop ()
+        """
+
+        if hasattr (self, '_orderprop') :
+            return self._orderprop
+        props = self.getprops ()
+        if props.has_key ('order'):
+            return 'order'
+        return self.labelprop ()
 
     def lookup(self, keyvalue):
         """Locate a particular node by its key property and return its id.
@@ -618,20 +690,20 @@ class Class:
         raise NotImplementedError
 
     def find(self, **propspec):
-        '''Get the ids of items in this class which link to the given items.
+        """Get the ids of nodes in this class which link to the given nodes.
 
-        'propspec' consists of keyword args propname=itemid or
-                   propname={<itemid 1>:1, <itemid 2>:1, ...}
+        'propspec' consists of keyword args propname={nodeid:1,}
         'propname' must be the name of a property in this class, or a
-                   KeyError is raised.  That property must be a Link or
-                   Multilink property, or a TypeError is raised.
+        KeyError is raised.  That property must be a Link or Multilink
+        property, or a TypeError is raised.
 
-        Any item in this class whose 'propname' property links to any of
-        the itemids will be returned. Examples::
+        Any node in this class whose 'propname' property links to any of the
+        nodeids will be returned. Used by the full text indexing, which knows
+        that "foo" occurs in msg1, msg3 and file7, so we have hits on these
+        issues:
 
-            db.issue.find(messages='1')
             db.issue.find(messages={'1':1,'3':1}, files={'7':1})
-        '''
+        """
         raise NotImplementedError
 
     def filter(self, search_matches, filterspec, sort=(None,None),
@@ -683,12 +755,35 @@ class Class:
         raise NotImplementedError
 
     def index(self, nodeid):
-        '''Add (or refresh) the node to search indexes
-        '''
+        """Add (or refresh) the node to search indexes"""
         raise NotImplementedError
 
+    #
+    # Detector interface
+    #
+    def audit(self, event, detector, priority = 100):
+        """Register an auditor detector"""
+        self.auditors[event].append((priority, detector))
+
+    def fireAuditors(self, event, nodeid, newvalues):
+        """Fire all registered auditors"""
+        for prio, audit in self.auditors[event]:
+            audit(self.db, self, nodeid, newvalues)
+
+    def react(self, event, detector, priority = 100):
+        """Register a reactor detector"""
+        self.reactors[event].append((priority, detector))
+
+    def fireReactors(self, event, nodeid, oldvalues):
+        """Fire all registered reactors"""
+        for prio, react in self.reactors[event]:
+            react(self.db, self, nodeid, oldvalues)
+
+    #
+    # import / export support
+    #
     def export_propnames(self):
-        '''List the property names for export from this Class.'''
+        """List the property names for export from this Class"""
         propnames = self.getprops().keys()
         propnames.sort()
         return propnames
@@ -768,29 +863,36 @@ class FileClass:
         propnames.sort()
         return propnames
 
+    def exportFilename(self, dirname, nodeid):
+        subdir_filename = self.db.subdirFilename(self.classname, nodeid)
+        return os.path.join(dirname, self.classname+'-files', subdir_filename)
+
     def export_files(self, dirname, nodeid):
         ''' Export the "content" property as a file, not csv column
         '''
         source = self.db.filename(self.classname, nodeid)
-        x, filename = os.path.split(source)
-        x, subdir = os.path.split(x)
-        dest = os.path.join(dirname, self.classname+'-files', subdir, filename)
-        if not os.path.exists(os.path.dirname(dest)):
-            os.makedirs(os.path.dirname(dest))
+
+        dest = self.exportFilename(dirname, nodeid)
+        ensureParentsExist(dest)
         shutil.copyfile(source, dest)
 
     def import_files(self, dirname, nodeid):
         ''' Import the "content" property as a file
         '''
+        source = self.exportFilename(dirname, nodeid)
+
         dest = self.db.filename(self.classname, nodeid, create=1)
-        x, filename = os.path.split(dest)
-        x, subdir = os.path.split(x)
-        source = os.path.join(dirname, self.classname+'-files', subdir,
-            filename)
-        if not os.path.exists(os.path.dirname(dest)):
-            os.makedirs(os.path.dirname(dest))
+        ensureParentsExist(dest)
         shutil.copyfile(source, dest)
 
+        mime_type = None
+        if self.getprops().has_key('type'):
+            mime_type = self.get(nodeid, 'type')
+        if not mime_type:
+            mime_type = self.default_mime_type
+        self.db.indexer.add_text((self.classname, nodeid, 'content'),
+            self.get(nodeid, 'content'), mime_type)
+
 class Node:
     ''' A convenience wrapper for the given node
     '''
