@@ -297,19 +297,6 @@ class Message(mimetools.Message):
 
 class MailGW:
 
-    # Matches subjects like:
-    # Re: "[issue1234] title of issue [status=resolved]"
-    subject_re = re.compile(r'''
-        (?P<refwd>\s*\W?\s*(fw|fwd|re|aw)\W\s*)*\s*   # Re:
-        (?P<quote>")?                                 # Leading "
-        (\[(?P<classname>[^\d\s]+)                    # [issue..
-           (?P<nodeid>\d+)?                           # ..1234]
-         \])?\s*
-        (?P<title>[^[]+)?                             # issue title
-        "?                                            # Trailing "
-        (\[(?P<args>.+?)\])?                          # [prop=value]
-        ''', re.IGNORECASE|re.VERBOSE)
-
     def __init__(self, instance, db, arguments=()):
         self.instance = instance
         self.db = db
@@ -611,6 +598,11 @@ Emails to Roundup trackers must include a Subject: line!
         # make local variable for easier access
         config = self.instance.config
 
+        # determine the sender's address
+        from_list = message.getaddrlist('resent-from')
+        if not from_list:
+            from_list = message.getaddrlist('from')
+
         # XXX Don't enable. This doesn't work yet.
 #  "[^A-z.]tracker\+(?P<classname>[^\d\s]+)(?P<nodeid>\d+)\@some.dom.ain[^A-z.]"
         # handle delivery to addresses like:tracker+issue25@some.dom.ain
@@ -627,21 +619,35 @@ Emails to Roundup trackers must include a Subject: line!
 #                    nodeid = issue.group('nodeid')
 #                    break
 
-        # determine the sender's address
-        from_list = message.getaddrlist('resent-from')
-        if not from_list:
-            from_list = message.getaddrlist('from')
+        # Matches subjects like:
+        # Re: "[issue1234] title of issue [status=resolved]"
+        open, close = config['MAILGW_SUBJECT_SUFFIX_DELIMITERS']
+        delim_open = re.escape(open)
+        delim_close = re.escape(close)
+        subject_re = re.compile(r'''
+        (?P<refwd>\s*\W?\s*(fw|fwd|re|aw)\W\s*)*\s*   # Re:
+        (?P<quote>")?                                 # Leading "
+        (\[(?P<classname>[^\d\s]+)                    # [issue..
+           (?P<nodeid>\d+)?                           # ..1234]
+         \])?\s*
+        (?P<title>[^%s]+)?                             # issue title
+        "?                                            # Trailing "
+        (?P<argswhole>%s(?P<args>.+?)%s)?             # [prop=value]
+        '''%(delim_open, delim_open, delim_close),
+        re.IGNORECASE|re.VERBOSE)
+
+        # figure subject line parsing modes
+        pfxmode = config['MAILGW_SUBJECT_PREFIX_PARSING']
+        sfxmode = config['MAILGW_SUBJECT_SUFFIX_PARSING']
 
         # check for well-formed subject line
-        m = self.subject_re.match(subject)
+        m = subject_re.match(subject)
         if m:
-            # get the classname
-            classname = m.group('classname')
-            if classname is None:
-                # no classname, check if this a registration confirmation email
-                # or fallback on the default class
+            # check for registration OTK
+            # or fallback on the default class
+            if self.db.config['EMAIL_REGISTRATION_CONFIRMATION']:
                 otk_re = re.compile('-- key (?P<otk>[a-zA-Z0-9]{32})')
-                otk = otk_re.search(m.group('title'))
+                otk = otk_re.search(m.group('title') or '')
                 if otk:
                     self.db.confirm_registration(otk.group('otk'))
                     subject = 'Your registration to %s is complete' % \
@@ -649,7 +655,13 @@ Emails to Roundup trackers must include a Subject: line!
                     sendto = [from_list[0][1]]
                     self.mailer.standard_message(sendto, subject, '')
                     return
-                elif self.default_class:
+            # get the classname
+            if pfxmode == 'none':
+                classname = None
+            else:
+                classname = m.group('classname')
+            if classname is None:
+                if self.default_class:
                     classname = self.default_class
                 else:
                     classname = config['MAILGW_DEFAULT_CLASS']
@@ -657,7 +669,7 @@ Emails to Roundup trackers must include a Subject: line!
                         # fail
                         m = None
 
-        if not m:
+        if not m and pfxmode == 'strict':
             raise MailUsageError, """
 The message you sent to roundup did not contain a properly formed subject
 line. The subject must contain a class name or designator to indicate the
@@ -672,10 +684,22 @@ line. The subject must contain a class name or designator to indicate the
 Subject was: '%s'
 """%subject
 
-        # get the class
-        try:
-            cl = self.db.getclass(classname)
-        except KeyError:
+        # try to get the class specified - if "loose" then fall back on the
+        # default
+        attempts = [classname]
+        if pfxmode == 'loose':
+            if self.default_class:
+                attempts.append(self.default_class)
+            else:
+                attempts.append(config['MAILGW_DEFAULT_CLASS'])
+        cl = None
+        for trycl in attempts:
+            try:
+                cl = self.db.getclass(classname)
+                break
+            except KeyError:
+                pass
+        if not cl:
             raise MailUsageError, '''
 The class name you identified in the subject line ("%s") does not exist in the
 database.
@@ -685,7 +709,10 @@ Subject was: "%s"
 '''%(classname, ', '.join(self.db.getclasses()), subject)
 
         # get the optional nodeid
-        nodeid = m.group('nodeid')
+        if pfxmode == 'none':
+            nodeid = None
+        else:
+            nodeid = m.group('nodeid')
 
         # title is optional too
         title = m.group('title')
@@ -703,7 +730,7 @@ Subject was: "%s"
         if nodeid is None and not title:
             raise MailUsageError, '''
 I cannot match your message to a node in the database - you need to either
-supply a full node identifier (with number, eg "[issue123]" or keep the
+supply a full designator (with number, eg "[issue123]" or keep the
 previous subject title intact so I can match that.
 
 Subject was: "%s"
@@ -713,20 +740,36 @@ Subject was: "%s"
         # maybe someone's responded to the initial mail that created an
         # entry. Try to find the matching nodes with the same title, and
         # use the _last_ one matched (since that'll _usually_ be the most
-        # recent...)
-        if nodeid is None and m.group('refwd'):
+        # recent...). The subject_content_match config may specify an
+        # additional restriction based on the matched node's creation or
+        # activity.
+        tmatch_mode = config['MAILGW_SUBJECT_CONTENT_MATCH']
+        if tmatch_mode != 'never' and nodeid is None and m.group('refwd'):
             l = cl.stringFind(title=title)
-            if l:
-                nodeid = l[-1]
+            limit = None
+            if (tmatch_mode.startswith('creation') or
+                    tmatch_mode.startswith('activity')):
+                limit, interval = tmatch_mode.split(' ', 1)
+                threshold = date.Date('.') - date.Interval(interval)
+            for id in l:
+                if limit:
+                    if threshold < cl.get(id, limit):
+                        nodeid = id
+                else:
+                    nodeid = id
 
         # if a nodeid was specified, make sure it's valid
         if nodeid is not None and not cl.hasnode(nodeid):
-            raise MailUsageError, '''
+            if pfxmode == 'strict':
+                raise MailUsageError, '''
 The node specified by the designator in the subject of your message ("%s")
 does not exist.
 
 Subject was: "%s"
 '''%(nodeid, subject)
+            else:
+                title = subject
+                nodeid = None
 
         # Handle the arguments specified by the email gateway command line.
         # We do this by looping over the list of self.arguments looking for
@@ -808,11 +851,14 @@ Unknown address: %s
 
         # make sure they're allowed to edit or create this class of information
         if nodeid:
-            if not self.db.security.hasPermission('Edit', author, classname):
-                raise Unauthorized, 'You are not permitted to edit %s.'%classname
+            if not self.db.security.hasPermission('Edit', author, classname,
+                    itemid=nodeid):
+                raise Unauthorized, 'You are not permitted to '\
+                    'edit %s.'%classname
         else:
             if not self.db.security.hasPermission('Create', author, classname):
-                raise Unauthorized, 'You are not permitted to create %s.'%classname
+                raise Unauthorized, 'You are not permitted to '\
+                    'create %s.'%classname
                 
         # the author may have been created - make sure the change is
         # committed before we reopen the database
@@ -848,17 +894,24 @@ Unknown address: %s
         properties = cl.getprops()
         props = {}
         args = m.group('args')
+        argswhole = m.group('argswhole')
         if args:
-            errors, props = setPropArrayFromString(self, cl, args, nodeid)
-            # handle any errors parsing the argument list
-            if errors:
-                errors = '\n- '.join(map(str, errors))
-                raise MailUsageError, '''
+            if sfxmode == 'none':
+                title += ' ' + argswhole
+            else:
+                errors, props = setPropArrayFromString(self, cl, args, nodeid)
+                # handle any errors parsing the argument list
+                if errors:
+                    if sfxmode == 'strict':
+                        errors = '\n- '.join(map(str, errors))
+                        raise MailUsageError, '''
 There were problems handling your subject line argument list:
 - %s
 
 Subject was: "%s"
 '''%(errors, subject)
+                    else:
+                        title += ' ' + argswhole
 
 
         # set the issue title to the subject
@@ -912,8 +965,10 @@ not find a text/plain part to use.
                 else:
                     files.append(fileid)
             # attach the files to the issue
-            if not self.db.security.hasPermission('Edit', author, classname, 'files'):
-                raise Unauthorized, 'You are not permitted to add files to %s.'%classname
+            if not self.db.security.hasPermission('Edit', author,
+                    classname, 'files'):
+                raise Unauthorized, 'You are not permitted to add '\
+                    'files to %s.'%classname
 
             if nodeid:
                 # extend the existing files list
@@ -942,8 +997,10 @@ Mail message was rejected by a detector.
 %s
 '''%error
             # attach the message to the node
-            if not self.db.security.hasPermission('Edit', author, classname, 'messages'):
-                raise Unauthorized, 'You are not permitted to add messages to %s.'%classname
+            if not self.db.security.hasPermission('Edit', author,
+                    classname, 'messages'):
+                raise Unauthorized, 'You are not permitted to add '\
+                    'messages to %s.'%classname
 
             if nodeid:
                 # add the message to the node's list
@@ -967,8 +1024,10 @@ Mail message was rejected by a detector.
 
             # Check permissions for each property
             for prop in props.keys():
-                if not self.db.security.hasPermission('Edit', author, classname, prop):
-                    raise Unauthorized, 'You are not permitted to edit property %s of class %s.'%(prop,classname)
+                if not self.db.security.hasPermission('Edit', author,
+                        classname, prop):
+                    raise Unauthorized, 'You are not permitted to edit '\
+                        'property %s of class %s.'%(prop,classname)
 
             if nodeid:
                 cl.set(nodeid, **props)
