@@ -56,7 +56,7 @@ class Database:
             return self.journal_uid[1]
 
     def setCurrentUser(self, username):
-        """Set the user that is responsible for current database 
+        """Set the user that is responsible for current database
         activities.
         """
         self.journaltag = username
@@ -65,7 +65,7 @@ class Database:
         """Check if a given username equals the already active user.
         """
         return self.journaltag == username
-    
+
     def getUserTimezone(self):
         """Return user timezone defined in 'timezone' property of user class.
         If no such property exists return 0
@@ -180,12 +180,13 @@ class IssueClass:
         list. These recipients will not be included in the To: or Cc:
         address lists.
         """
-        if msgid is None:
-            authid = '1'
-            recipients = []
-        else:
+        if msgid:
             authid = self.db.msg.get(msgid, 'author')
             recipients = self.db.msg.get(msgid, 'recipients', [])
+        else:
+            # "system message"
+            authid = None
+            recipients = []
 
         sendto = []
         bcc_sendto = []
@@ -274,15 +275,18 @@ class IssueClass:
         title = self.get(nodeid, 'title') or '%s message copy'%cn
 
         # figure author information
-        if msgid is None:
-            authid = '1'
-            authname = 'admin'
-        else:
+        if msgid:
             authid = messages.get(msgid, 'author')
             authname = users.get(authid, 'realname')
-        if not authname:
-            authname = users.get(authid, 'username', '')
-        authaddr = users.get(authid, 'address', '')
+            if not authname:
+                authname = users.get(authid, 'username', '')
+            authaddr = users.get(authid, 'address', '')
+        else:
+            # "system message"
+            authid = None
+            authname = 'admin'
+            authaddr = self.db.config.ADMIN_EMAIL
+
         if authaddr:
             authaddr = " <%s>" % straddr( ('',authaddr) )
 
@@ -346,60 +350,88 @@ class IssueClass:
         subject = '[%s%s] %s'%(cn, nodeid, title)
         author = (authname + from_tag, from_address)
 
-        # create the message
-        mailer = Mailer(self.db.config)
-        message, writer = mailer.get_standard_message(sendto, subject, author)
-
-        # set reply-to to the tracker
-        tracker_name = self.db.config.TRACKER_NAME
-        if charset != 'utf-8':
-            tracker = unicode(tracker_name, 'utf-8').encode(charset)
-        tracker_name = encode_header(tracker_name, charset)
-        writer.addheader('Reply-To', straddr((tracker_name, from_address)))
-
-        # message ids
-        if messageid:
-            writer.addheader('Message-Id', messageid)
-        if inreplyto:
-            writer.addheader('In-Reply-To', inreplyto)
-
-        # attach files
-        if message_files:
-            part = writer.startmultipartbody('mixed')
-            part = writer.nextpart()
-            part.addheader('Content-Transfer-Encoding', 'quoted-printable')
-            body = part.startbody('text/plain; charset=%s'%charset)
-            body.write(content_encoded)
-            for fileid in message_files:
-                name = files.get(fileid, 'name')
-                mime_type = files.get(fileid, 'type')
-                content = files.get(fileid, 'content')
-                part = writer.nextpart()
-                if mime_type == 'text/plain':
-                    part.addheader('Content-Disposition',
-                        'attachment;\n filename="%s"'%name)
-                    part.addheader('Content-Transfer-Encoding', '7bit')
-                    body = part.startbody('text/plain')
-                    body.write(content)
-                else:
-                    # some other type, so encode it
-                    if not mime_type:
-                        # this should have been done when the file was saved
-                        mime_type = mimetypes.guess_type(name)[0]
-                    if mime_type is None:
-                        mime_type = 'application/octet-stream'
-                    part.addheader('Content-Disposition',
-                        'attachment;\n filename="%s"'%name)
-                    part.addheader('Content-Transfer-Encoding', 'base64')
-                    body = part.startbody(mime_type)
-                    body.write(base64.encodestring(content))
-            writer.lastpart()
+        # send an individual message per recipient?
+        if self.db.config.NOSY_EMAIL_SENDING != 'single':
+            sendto = [[address] for address in sendto]
         else:
-            writer.addheader('Content-Transfer-Encoding', 'quoted-printable')
-            body = writer.startbody('text/plain; charset=%s'%charset)
-            body.write(content_encoded)
+            sendto = [sendto]
 
-        mailer.smtp_send(sendto + bcc_sendto, message)
+        # now send one or more messages
+        # TODO: I believe we have to create a new message each time as we
+        # can't fiddle the recipients in the message ... worth testing
+        # and/or fixing some day
+        first = True
+        for sendto in sendto:
+            # create the message
+            mailer = Mailer(self.db.config)
+            message, writer = mailer.get_standard_message(sendto, subject,
+                author)
+
+            # set reply-to to the tracker
+            tracker_name = self.db.config.TRACKER_NAME
+            if charset != 'utf-8':
+                tracker = unicode(tracker_name, 'utf-8').encode(charset)
+            tracker_name = encode_header(tracker_name, charset)
+            writer.addheader('Reply-To', straddr((tracker_name, from_address)))
+
+            # message ids
+            if messageid:
+                writer.addheader('Message-Id', messageid)
+            if inreplyto:
+                writer.addheader('In-Reply-To', inreplyto)
+
+            # attach files
+            if message_files:
+                part = writer.startmultipartbody('mixed')
+                part = writer.nextpart()
+                part.addheader('Content-Transfer-Encoding', 'quoted-printable')
+                body = part.startbody('text/plain; charset=%s'%charset)
+                body.write(content_encoded)
+                for fileid in message_files:
+                    name = files.get(fileid, 'name')
+                    mime_type = files.get(fileid, 'type')
+                    content = files.get(fileid, 'content')
+                    part = writer.nextpart()
+                    if mime_type == 'text/plain':
+                        part.addheader('Content-Disposition',
+                            'attachment;\n filename="%s"'%name)
+                        try:
+                            content.decode('ascii')
+                        except UnicodeError:
+                            # the content cannot be 7bit-encoded.
+                            # use quoted printable
+                            part.addheader('Content-Transfer-Encoding',
+                                'quoted-printable')
+                            body = part.startbody('text/plain')
+                            body.write(quopri.encodestring(content))
+                        else:
+                            part.addheader('Content-Transfer-Encoding', '7bit')
+                            body = part.startbody('text/plain')
+                            body.write(content)
+                    else:
+                        # some other type, so encode it
+                        if not mime_type:
+                            # this should have been done when the file was saved
+                            mime_type = mimetypes.guess_type(name)[0]
+                        if mime_type is None:
+                            mime_type = 'application/octet-stream'
+                        part.addheader('Content-Disposition',
+                            'attachment;\n filename="%s"'%name)
+                        part.addheader('Content-Transfer-Encoding', 'base64')
+                        body = part.startbody(mime_type)
+                        body.write(base64.encodestring(content))
+                writer.lastpart()
+            else:
+                writer.addheader('Content-Transfer-Encoding',
+                    'quoted-printable')
+                body = writer.startbody('text/plain; charset=%s'%charset)
+                body.write(content_encoded)
+
+            if first:
+                mailer.smtp_send(sendto + bcc_sendto, message)
+            else:
+                mailer.smtp_send(sendto, message)
+            first = False
 
     def email_signature(self, nodeid, msgid):
         ''' Add a signature to the e-mail with some useful information
@@ -554,4 +586,4 @@ class IssueClass:
             m.insert(0, '')
         return '\n'.join(m)
 
-# vim: set filetype=python ts=4 sw=4 et si
+# vim: set filetype=python sts=4 sw=4 et si :
