@@ -17,7 +17,7 @@
 #
 # $Id$
 
-import unittest, os, shutil, errno, imp, sys, time, pprint
+import unittest, os, shutil, errno, imp, sys, time, pprint, sets
 
 from roundup.hyperdb import String, Password, Link, Multilink, Date, \
     Interval, DatabaseError, Boolean, Number, Node
@@ -26,7 +26,7 @@ from roundup import date, password, init, instance, configuration
 from mocknull import MockNull
 
 config = configuration.CoreConfig()
-config.DATABASE = "_test_dir"
+config.DATABASE = "db"
 config.RDBMS_NAME = "rounduptest"
 config.RDBMS_HOST = "localhost"
 config.RDBMS_USER = "rounduptest"
@@ -79,6 +79,8 @@ def setupSchema(db, create, module):
         priority=Link('priority'))
     stuff = module.Class(db, "stuff", stuff=String())
     session = module.Class(db, 'session', title=String())
+    msg = module.FileClass(db, "msg",
+                           author=Link("user", do_journal='no'))
     session.disableJournalling()
     db.post_init()
     if create:
@@ -396,18 +398,36 @@ class DBTest(MyTestCase):
         self.db.issue.create(title="spam", status='1')
         b = self.db.status.get('1', 'name')
         a = self.db.status.list()
+        nodeids = self.db.status.getnodeids()
         self.db.status.retire('1')
+        others = nodeids[:]
+        others.remove('1')
+
+        self.assertEqual(sets.Set(self.db.status.getnodeids()),
+            sets.Set(nodeids))
+        self.assertEqual(sets.Set(self.db.status.getnodeids(retired=True)),
+            sets.Set(['1']))
+        self.assertEqual(sets.Set(self.db.status.getnodeids(retired=False)),
+            sets.Set(others))
+
+        self.assert_(self.db.status.is_retired('1'))
+
         # make sure the list is different
         self.assertNotEqual(a, self.db.status.list())
+
         # can still access the node if necessary
         self.assertEqual(self.db.status.get('1', 'name'), b)
         self.assertRaises(IndexError, self.db.status.set, '1', name='hello')
         self.db.commit()
+        self.assert_(self.db.status.is_retired('1'))
         self.assertEqual(self.db.status.get('1', 'name'), b)
         self.assertNotEqual(a, self.db.status.list())
+
         # try to restore retired node
         self.db.status.restore('1')
 
+        self.assert_(not self.db.status.is_retired('1'))
+
     def testCacheCreateSet(self):
         self.db.issue.create(title="spam", status='1')
         a = self.db.issue.get('1', 'title')
@@ -715,7 +735,7 @@ class DBTest(MyTestCase):
         # unindexed stopword
         self.assertEquals(self.db.indexer.search(['the'], self.db.issue), {})
 
-    def testReindexing(self):
+    def testReindexingChange(self):
         search = self.db.indexer.search
         issue = self.db.issue
         i1 = issue.create(title="flebble plop")
@@ -730,6 +750,15 @@ class DBTest(MyTestCase):
         self.assertEquals(search(['plop'], issue), {i1: {}})
         self.assertEquals(search(['flebble'], issue), {i2: {}})
 
+    def testReindexingClear(self):
+        search = self.db.indexer.search
+        issue = self.db.issue
+        i1 = issue.create(title="flebble plop")
+        i2 = issue.create(title="flebble frooz")
+        self.db.commit()
+        self.assertEquals(search(['plop'], issue), {i1: {}})
+        self.assertEquals(search(['flebble'], issue), {i1: {}, i2: {}})
+
         # unset i1's title
         issue.set(i1, title="")
         self.db.commit()
@@ -967,6 +996,7 @@ class DBTest(MyTestCase):
         # may fail :)
         ae(filt(None, {'deadline': 'from 2003-02-16'}), ['1', '3', '4'])
         ae(filt(None, {'deadline': '2003-02-16;'}), ['1', '3', '4'])
+        ae(filt(None, {'deadline': '2003-02-16;'}), ['1', '3', '4'])
         # year and month granularity
         ae(filt(None, {'deadline': '2002'}), [])
         ae(filt(None, {'deadline': '2003'}), ['1', '2', '3'])
@@ -981,6 +1011,17 @@ class DBTest(MyTestCase):
         ae(filt(None, {'foo': 'from 5:50'}), ['2'])
         ae(filt(None, {'foo': 'to 0:05'}), [])
 
+        # further
+        for issue in (
+                { 'deadline': date.Date('. -2d')},
+                { 'deadline': date.Date('. -1d')},
+                { 'deadline': date.Date('. -8d')},
+                ):
+            self.db.issue.create(**issue)
+        ae(filt(None, {'deadline': '-2d;'}), ['5', '6'])
+        ae(filt(None, {'deadline': '-1d;'}), ['6'])
+        ae(filt(None, {'deadline': '-1w;'}), ['5', '6'])
+
     def testFilteringIntervalSort(self):
         # 1: '1:10'
         # 2: '1d'
@@ -1001,6 +1042,15 @@ class DBTest(MyTestCase):
         ae(filt(None, {}, ('+','nosy'), (None,None)), ['1', '2', '4', '3'])
         ae(filt(None, {}, ('-','nosy'), (None,None)), ['3', '4', '1', '2'])
 
+    def testFilteringLinkSortGroup(self):
+        # 1: status: 2, priority: 3
+        # 2: status: 1, priority: 3
+        # 3: status: 1, priority: 2,
+        # 4: status: 3, priority: 2,
+        ae, filt = self.filteringSetup()
+        ae(filt(None, {}, ('+','status'), ('+','priority')),
+            ['1', '2', '4', '3'])
+
     def testFilteringDateSort(self):
         # '1': '2003-02-16.22:50'
         # '2': '2003-01-01.00:00'
@@ -1079,7 +1129,7 @@ class DBTest(MyTestCase):
                 for itemprops in items[1:]:
                     id = int(klass.import_list(names, itemprops))
                     if hasattr(klass, 'import_files'):
-                        klass.import_files('_test_export', id)
+                        klass.import_files('_test_export', str(id))
                     maxid = max(maxid, id)
                 self.db.setid(cn, str(maxid+1))
                 klass.import_journals(journals[cn])
@@ -1317,6 +1367,20 @@ class SchemaTest(MyTestCase):
         # confirm journal's ok
         self.db.getjournal('a', aid)
 
+    def test_removeClassKey(self):
+        self.init_amod()
+        aid = self.db.a.create(name='apple')
+        self.assertEqual(self.db.a.lookup('apple'), aid)
+        self.db.commit(); self.db.close()
+
+        self.db = self.module.Database(config, 'admin')
+        a = self.module.Class(self.db, "a", name=String(), newstr=String())
+        self.db.post_init()
+
+        aid2 = self.db.a.create(name='apple', newstr='booz')
+        self.db.commit()
+
+
     def init_amodml(self):
         self.db = self.module.Database(config, 'admin')
         a = self.module.Class(self.db, "a", name=String(),
